/*
*  RAL -- Rubrica Addressbook Library
*  file: abook.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <time.h>
#include <libintl.h>
#include <glib.h>
#include <glib-object.h>
#include <glib/gstdio.h>
#include <libxml/tree.h>
#include <libxml/parser.h>

#include "abook.h"
#include "card.h"
#include "personal.h"
#include "company.h"
#include "io.h"
#include "load.h"
#include "save.h"
#include "utils.h"
#include "error.h"

#define RUBRICA_FILE_FORMAT 4
// #define VERSION 4


enum {
  RUBRICA_ABOOK_HAVE_NAME = 1,
  RUBRICA_ABOOK_NAME,
  RUBRICA_ABOOK_PATH,
  RUBRICA_ABOOK_LAST
};



static gint addressbook_id = 0;


struct _RubricaAbookPrivate {
  gboolean have_name;   
  
  gint   abook_id;      /* addressbook's id                */
  gchar* name;          /* the addressbook (the file) name */
  gchar* path;          /* the addressbook (the file) path */
  gchar* id;            /* selected card id                */

  GList* trash;

  GList* cards;
  GList* iter;

  gboolean dispose_has_run;
};


/*    signals enumeration 
 */
enum {
  OPEN_FAIL,                /* file not opened     */
  SAVE_FAIL,                /* file not saved      */

  ADDRESSBOOK_CHANGED,      /* addressbook changed */
  ADDRESSBOOK_READ,         /* file read           */ 
  ADDRESSBOOK_SAVED,        /* file saved          */
  
  CARD_READ,                /* card read           */ 
  CARD_ADDED,              
  CARD_REMOVED,           
  CARD_REPLACED,
  
  LAST_SIGNAL 
};

static guint rubrica_abook_signals[LAST_SIGNAL] = {0};



static void rubrica_abook_class_init (RubricaAbookClass* klass);
static void rubrica_abook_init       (RubricaAbook* obj);

static void rubrica_abook_dispose    (RubricaAbook* obj);
static void rubrica_abook_finalize   (RubricaAbook* obj);


static gboolean rubrica_abook_open_rubrica_file (RubricaAbook* self, 
						 gchar* filename);
static gboolean rubrica_abook_parse_doc         (RubricaAbook* self,
						 xmlDocPtr xmldoc);

static gboolean rubrica_abook_save_rubrica_file (RubricaAbook* self, 
						 gchar* filename);
static gboolean rubrica_abook_write_file        (RubricaAbook* abook, 
						 gchar* filename);


gboolean opening_old = FALSE;


GType
rubrica_abook_get_type()
{
  static GType rubrica_abook_type = 0;
  
  if (!rubrica_abook_type)
    {
      static const GTypeInfo rubrica_abook_info =
	{
	  sizeof(RubricaAbookClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_abook_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaAbook),
	  0,
	  (GInstanceInitFunc) rubrica_abook_init
	};

      rubrica_abook_type = g_type_register_static (G_TYPE_OBJECT, 
						   "RubricaAbook",
						   &rubrica_abook_info, 0);
    }
  
  return rubrica_abook_type;
}


static void
rubrica_abook_class_init(RubricaAbookClass* klass)
{
  GObjectClass *class;
  //  GParamSpec* pspec;
  
  class  = G_OBJECT_CLASS (klass);

  class->dispose      = (GObjectFinalizeFunc) rubrica_abook_dispose;
  class->finalize     = (GObjectFinalizeFunc) rubrica_abook_finalize;

  klass->abook_open_file = rubrica_abook_open_rubrica_file;
  klass->abook_save_file = rubrica_abook_save_rubrica_file;
  klass->abook_export    = NULL;

  /* class signals 
   */ 

/**
 * RubricaAbook::open-fail:
 * @abook: the #RubricaAbook object that receives the signal
 * @err: an #RubricaError
 *
 * the "open-fail" signal is emitted if given rubrica's file can't be read
 */
  rubrica_abook_signals[OPEN_FAIL] =
    g_signal_new("open_fail", 
		 RUBRICA_ABOOK_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RubricaAbookClass, open_fail),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__INT,
		 G_TYPE_NONE,            /* return type */
		 1,                      /* params      */
		 G_TYPE_INT);            /* params type: error code */



/**
 * RubricaAbook::save-fail:
 * @abook: the #RubricaAbook object that receives the signal
 * @err: an #RubricaError
 *
 * the "save-fail" signal is emitted if given rubrica's file can't be read 
 */
  rubrica_abook_signals[SAVE_FAIL] =
    g_signal_new("save_fail", 
		 RUBRICA_ABOOK_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RubricaAbookClass, save_fail),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__INT,
		 G_TYPE_NONE,            /* return type */
		 1,                      /* params      */
		 G_TYPE_INT);            /* params type: error code */


/**
 * RubricaAbook::card-read:
 * @abook: the #RubricaAbook object that receives the signal
 * @data: a pointer to the readed card 
 * 
 * The "card-read" signal is emitted after the addressbook has been readed
 */
  rubrica_abook_signals[CARD_READ] = 
    g_signal_new("card_read", 
		 RUBRICA_ABOOK_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RubricaAbookClass, card_read),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__POINTER,
		 G_TYPE_NONE,
		 1,
		 G_TYPE_POINTER);
  
  
  /**
   * RubricaAbook::card-added:
   * @abook: the #RubricaAbook object that receives the signal
   * @data: a pointer to the readed card 
   *
   * The "card-added" signal is emitted after a card is added to addressbook.
   * A pointer to the card is passed to the callback function
   */
  rubrica_abook_signals[CARD_ADDED] =
    g_signal_new("card_added", 
		 RUBRICA_ABOOK_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RubricaAbookClass, card_added),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__POINTER,
		 G_TYPE_NONE,
		 1,
		 G_TYPE_POINTER);
  
  /**
   * RubricaAbook::card-removed:
   * @abook: the #RubricaAbook object that receives the signal
   *
   * The "card-removed" signal is emitted when a card is 
   * removed from addressbook
   */
  rubrica_abook_signals[CARD_REMOVED] =
    g_signal_new("card_removed", 
		 RUBRICA_ABOOK_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RubricaAbookClass, card_removed),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__STRING,
		 G_TYPE_NONE,
		 1,
		 G_TYPE_POINTER);

  /**
   * RubricaAbook::card-replaced:
   * @abook: the #RubricaAbook object that receives the signal
   * @data: a pointer to the readed card 
   *
   * The "card-replaced" signal is emitted a card replacement.
   * A pointer to the new card is passed to the callback function
   */
  rubrica_abook_signals[CARD_REPLACED] =
    g_signal_new("card_replaced", 
		 RUBRICA_ABOOK_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RubricaAbookClass, card_replaced),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__POINTER,
		 G_TYPE_NONE,
		 1,
		 G_TYPE_POINTER);


  /**
   * RubricaAbook::addressbook-read:
   * @abook: the #RubricaAbook object that receives the signal
   *
   * The "addressbook-read" signal is emitted when the 
   * addressbook is read
   */
  rubrica_abook_signals[ADDRESSBOOK_READ] =
    g_signal_new("addressbook_read", 
		 RUBRICA_ABOOK_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RubricaAbookClass, addressbook_read),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__VOID,
		 G_TYPE_NONE,
		 0,
		 G_TYPE_NONE);


  /**
   * RubricaAbook::addressbook_saved:
   * @abook: the #RubricaAbook object that receives the signal
   *
   * The "addressbook_saved" signal
   */
  rubrica_abook_signals[ADDRESSBOOK_SAVED] =
    g_signal_new("addressbook_saved", 
		 RUBRICA_ABOOK_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RubricaAbookClass, addressbook_saved),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__VOID,
		 G_TYPE_NONE,
		 0,
		 G_TYPE_NONE);

  /**
   * RubricaAbook::addressbook_changed:
   * @abook: the #RubricaAbook object that receives the signal
   *
   * The "addressbook_changed" signal
   */
  rubrica_abook_signals[ADDRESSBOOK_CHANGED] =
    g_signal_new("addressbook_changed", 
		 RUBRICA_ABOOK_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RubricaAbookClass, addressbook_changed),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__VOID,
		 G_TYPE_NONE,
		 0,
		 G_TYPE_NONE);


}


static void
rubrica_abook_init(RubricaAbook* self)
{
  self->private = g_new(RubricaAbookPrivate, 1);

  self->private->have_name = FALSE;
  self->private->abook_id  = addressbook_id++;
  self->private->name      = g_strdup(gettext("no name"));
  self->private->path      = NULL;
  self->private->id        = NULL;
 
  self->private->cards     = NULL;
  self->private->trash     = NULL;
  self->private->iter      = NULL;

  self->private->dispose_has_run = FALSE;
}

 

static void 
rubrica_abook_dispose (RubricaAbook* self)
{
  g_return_if_fail(IS_RUBRICA_ABOOK(self));
  
  /* free the list's items (PersonalCard or CompanyCard) 
   */
  self->private->iter = self->private->cards;
  for (; self->private->iter; self->private->iter = self->private->iter->next)
    {
      RubricaCard* card = self->private->iter->data;

      if (IS_RUBRICA_CARD(card))
	rubrica_card_free(RUBRICA_CARD(card));
    }
  
  g_list_free(self->private->cards);
  self->private->cards = NULL;
  self->private->iter  = NULL;
  
  if (self->private->dispose_has_run)
    return;

  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_abook_finalize (RubricaAbook* self)
{
  g_return_if_fail(IS_RUBRICA_ABOOK(self));
  
  rubrica_utils_free_string(self->private->name);
  rubrica_utils_free_string(self->private->path);

  g_free(self->private);
  self->private = NULL;
}


static gboolean 
rubrica_abook_open_rubrica_file (RubricaAbook* self, gchar* filename) 
{ 
  xmlDocPtr xmldoc = NULL;
  RubricaError err; 
  gboolean loaded; 
  gint fileformat  = 0;

  g_return_val_if_fail(IS_RUBRICA_ABOOK(self), FALSE);
  
  if (!filename)
    {
      g_signal_emit_by_name(self, "open_fail", NO_FILENAME, G_TYPE_INT);

      return FALSE;
    }

  if (!g_file_test(filename, G_FILE_TEST_EXISTS))
    {
      g_signal_emit_by_name(self, "open_fail", FILE_NOT_EXIST, G_TYPE_INT);

      return FALSE;
    }

  if((xmldoc = rubrica_load_open_doc(filename, &err)) == NULL)
    {
      g_signal_emit_by_name(self, "open_fail", err, G_TYPE_INT);

      return FALSE;
    }

  if(!rubrica_load_is_rubrica_doc(xmldoc, &err))
    {
      g_signal_emit_by_name(self, "open_fail", err, G_TYPE_INT);
      
      xmlFreeDoc(xmldoc);
      return FALSE;
    }

  /* get the file format
   */
  fileformat = rubrica_load_get_fileformat(xmldoc, &err);
  if (fileformat < RUBRICA_FILE_FORMAT)  /* this is an old rubrica file */
    {
      /* close opened file 
      xmlFreeDoc(xmldoc);
      */
      if (fileformat == 3)
	{
	  /* error signals emitted in rubrica_xcard_load_ff3_doc
	  if (!rubrica_xcard_load_ff3_doc(xcard, fname))
	    return FALSE;	  
	  else
	    return TRUE;
	  */
	}      
      /*
      if (!rubrica_xcard_io_import_old_file(filename))
	return FALSE;
      */
      /*
      if((xmldoc = rubrica_io_open_doc(filename, &err)) == NULL)
	return FALSE;
      
      if(!rubrica_xio_is_rubrica_doc(xmldoc, &err))
	return NULL;     
      */
    }

  loaded = rubrica_abook_parse_doc(self, xmldoc);

  if (!loaded)
    {
      g_signal_emit_by_name(self, "open_fail", FILE_NOT_OPENED, G_TYPE_INT);
      
      xmlFreeDoc(xmldoc);

      return FALSE;
    }
  else
    g_signal_emit_by_name(self, "addressbook_read", NULL, G_TYPE_NONE);

  xmlFreeDoc(xmldoc);

  rubrica_abook_set_name(self, g_path_get_basename(filename));
  rubrica_abook_set_path(self, g_path_get_dirname(filename));
  
  return TRUE;
}


static gboolean 
rubrica_abook_parse_doc(RubricaAbook* self, xmlDocPtr xmldoc)
{  
  xmlNodePtr xmlabook;
  RubricaError err;
     
  g_return_val_if_fail(IS_RUBRICA_ABOOK(self), FALSE);
  g_return_val_if_fail(xmldoc != NULL, FALSE);
  
  xmlabook = xmldoc->children;
  if (xmlIsBlankNode(xmlabook))
    xmlabook = xmlabook->next;
  
  if (xmlabook && xmlabook->name && xmlStrcmp(xmlabook->name,
					      (xmlChar*) "Rubrica") == 0) 
    {
      xmlabook = xmlabook->children;
      
      if (xmlIsBlankNode(xmlabook))
	xmlabook = xmlabook->next;

      while (xmlabook)
	{
	  xmlNodePtr xmlcard = NULL;

	  xmlcard = rubrica_io_get_node(xmlabook, (xmlChar*) "Card");
	  
	  if (xmlcard)
	    {
	      gchar* tmp = NULL;
	      
	      tmp = rubrica_io_get_prop(xmlcard, "type", &err);
	      if (!tmp)
		tmp = g_strdup("personal");
	      if (g_ascii_strcasecmp("personal", tmp) == 0)
		{
		  RubricaPersonalCard *card;  

		  card = rubrica_load_read_personal_xmlcard(xmlcard);  
		  if (card)
		    {
		      rubrica_abook_add_card(self, (gpointer) card); 
		      g_signal_emit_by_name(self, "card_read", card, 
					    G_TYPE_POINTER);
		    }
		}
	      else if (g_ascii_strcasecmp("company", tmp) == 0)
		{
		  RubricaCompanyCard* card;

		  card = rubrica_load_read_company_xmlcard(xmlcard);

		  if (card)
		    {
		      rubrica_abook_add_card(self, (gpointer) card);
		      
		      g_signal_emit_by_name(self, "card_read", card, 
					    G_TYPE_POINTER);
		    }
		}
	      
	      g_free(tmp);	   	      	      	     
	    }  

	   /* parse next Card */
	   xmlabook = xmlabook->next; 
	   if(xmlIsBlankNode(xmlabook))
	     xmlabook = xmlabook->next;
	}
    }
  else
    {
      g_signal_emit_by_name(self, "open_fail", NO_RUBRICA_DOC, G_TYPE_INT);
      
      return FALSE;
    }

  g_signal_emit_by_name(self, "addressbook_read", NULL, G_TYPE_NONE);

  return TRUE;
}



static gboolean 
rubrica_abook_save_rubrica_file (RubricaAbook* self, gchar* filename)
{
  g_return_val_if_fail(IS_RUBRICA_ABOOK(self), FALSE);
    
  if (!filename)
    {
      g_signal_emit_by_name(self, "save_fail", NO_FILENAME, G_TYPE_INT);

      return FALSE;
    }

  if (g_file_test(filename, G_FILE_TEST_EXISTS))
    {
      g_signal_emit_by_name(self, "save_fail", FILE_EXIST, G_TYPE_INT);

      return FALSE;
    }

  if (!rubrica_abook_write_file(self, filename))
    {
      g_signal_emit_by_name(self, "save_fail", WRITING_XML, G_TYPE_INT);
      
      return FALSE;
    }
    
  g_signal_emit_by_name(self, "addressbook_saved", NULL, G_TYPE_NONE);
 
  return TRUE;
}



static gboolean
rubrica_abook_write_file(RubricaAbook* self, gchar* name)
{
  xmlDocPtr doc;        /* puntatore al documento xml */
  gpointer card;
  gchar *path = NULL, *fname;
  gchar *fileformat;

  g_return_val_if_fail(IS_RUBRICA_ABOOK(self), FALSE);
  g_return_val_if_fail(name != NULL, FALSE);

  path  = g_dirname(name);
  fname = g_strdup(name);

  /* creazione di un nuovo documento xml 
   */
  doc = xmlNewDoc((xmlChar*) "1.0");
  
  /* creo un nuovo nodo radice rubrica 
   */
  fileformat = g_strdup_printf("%d", RUBRICA_FILE_FORMAT);

  doc->children = xmlNewDocNode(doc, NULL, (xmlChar*) "Rubrica", NULL);
  xmlSetProp(doc->children, (xmlChar*) "version", (xmlChar*) RUBRICA_VERSION);
  xmlSetProp(doc->children, (xmlChar*) "fileformat", (xmlChar*) fileformat); 
  xmlSetProp(doc->children, (xmlChar*) "doctype", (xmlChar*) "AddressBook"); 

  g_free(fileformat);

  /* attraverso la rubrica e scrivo i dati 
     nell'albero xml
  */
  rubrica_abook_reset_book(self);
  card = rubrica_abook_get_card(self);
  for ( ; card; card = rubrica_abook_get_next_card(self))
    {
      xmlNodePtr cardxml;     /* card node */

      cardxml = xmlNewChild(doc->children, NULL, 
			    (xmlChar*) "Card", (xmlChar*) NULL);
      
      rubrica_save_write_card(RUBRICA_CARD(card), cardxml);
    }     
  
  if (xmlSaveFormatFile(fname, doc, 1) == -1) 
    return FALSE;

  /* if we are here the addressbook is write 
   */  
  xmlFreeDoc(doc);    
  g_free(fname);

  return TRUE;
}



/*   ***************************** Public *****************************
*/


/**
 * rubrica_abook_new
 *
 * create a #RubricaAbook
 *
 * Returns: a new #RubricaAbook
 */
RubricaAbook*
rubrica_abook_new(void)
{
  RubricaAbook* abook;

  abook = g_object_new(rubrica_abook_get_type(), NULL);
  
  return abook;
}


/**
 * rubrica_abook_free
 * @abook: a #RubricaAbook
 * 
 * free the #RubricaAbook
 */
void
rubrica_abook_free(RubricaAbook* abook)
{
  g_return_if_fail(IS_RUBRICA_ABOOK(abook));

  g_object_unref(abook);   
}



/**
 * rubrica_abook_open_file
 * @abook: a #RubricaAbook
 * @fname: the name of the addressbook
 *
 * open the addressbook with given name
 *
 * Returns: %TRUE if file is opened successfully, %FALSE otherwise
 */
gboolean 
rubrica_abook_open_file (RubricaAbook* abook, gchar* fname)
{
  RubricaAbookClass* class;

  g_return_val_if_fail(IS_RUBRICA_ABOOK(abook), FALSE);

  class = RUBRICA_ABOOK_GET_CLASS(abook);
  
  if (class->abook_open_file)
    return class->abook_open_file(abook, fname); 

  return FALSE;
}



/**
 * rubrica_abook_save_file
 * @abook: a #RubricaAbook
 * @fname: the name of file
 * 
 * save the addressbook with given filename
 *
* Returns: %TRUE if file is saved successfully, %FALSE otherwise
 */
gboolean 
rubrica_abook_save_file (RubricaAbook* abook, gchar* fname)
{
  RubricaAbookClass* class;

  g_return_val_if_fail(IS_RUBRICA_ABOOK(abook), FALSE);

  class = RUBRICA_ABOOK_GET_CLASS(abook);
  
  if (class->abook_save_file)
    return class->abook_save_file(abook, fname); 
  
  return FALSE;
}



gboolean
rubrica_abook_overwrite_file(RubricaAbook* abook)
{
  gchar *oldfile, *tmp;
  
  oldfile = rubrica_abook_get_name(abook);
  tmp = g_strdup_printf("%s~", oldfile);
  g_rename(oldfile, tmp);
  g_remove(oldfile);
  g_free(tmp);
  
  return rubrica_abook_write_file(abook, oldfile);
}


/**
 * rubrica_abook_copy
 * @abook: a #RubricaAbook
 *
 * copy the addressbook pointed by abook
 *
 * Returns: a new #RubricaAbook
 */
RubricaAbook* 
rubrica_abook_copy (RubricaAbook* abook)
{
  RubricaAbook* new;
  gpointer card;

  g_return_val_if_fail(IS_RUBRICA_ABOOK(abook), NULL);
  
  new = rubrica_abook_new();
  
  card = rubrica_abook_get_card(abook);
  for (; card; card = rubrica_abook_get_next_card(abook))
    {
      gpointer new_card = NULL; 

      new_card = rubrica_card_copy(RUBRICA_CARD(card));
      
      if (new_card)
	rubrica_abook_add_card(new, new_card);
    }    
  rubrica_abook_reset_book(abook);
  
  return new;
}



/**
 * rubrica_abook_set_name
 * @abook: a #RubricaAbook
 * @name: the name of the addressbook
 * 
 * set the name of addressbook
 */
void   
rubrica_abook_set_name (RubricaAbook* abook, gchar* name)
{
  g_return_if_fail(IS_RUBRICA_ABOOK(abook)); 

  rubrica_utils_free_string(abook->private->name);
  abook->private->name = g_strdup(name);
}



/**
 * rubrica_abook_set_path
 * @abook: a #RubricaAbook
 * @path: the addressbook's path
 * 
 * set the path of addressbook
 */
void   
rubrica_abook_set_path (RubricaAbook* abook, gchar* path)
{
  g_return_if_fail(IS_RUBRICA_ABOOK(abook));

  rubrica_utils_free_string(abook->private->path);
  abook->private->path = g_strdup(path);  
}


/**
 * rubrica_abook_set_selected_card_id
 * @abook: a #RubricaAbook
 * @id: the card's id 
 *  
 * store the id of currently active card
 */
void          
rubrica_abook_set_selected_card_id (RubricaAbook* abook, gchar* id)
{
  g_return_if_fail(IS_RUBRICA_ABOOK(abook));

  rubrica_utils_free_string(abook->private->id);
  abook->private->id = g_strdup(id);
}


/**
 * rubrica_abook_get_name
 * @abook: a #RubricaAbook
 *
 * get the address book's name. Caller must free the returned value
 *
 * Returns: a gchar* 
 */
gchar* 
rubrica_abook_get_name (RubricaAbook* abook)
{
  g_return_val_if_fail(IS_RUBRICA_ABOOK(abook), NULL);

  return g_strdup(abook->private->name);  
}




/**
 * rubrica_abook_get_path
 * @abook: a #RubricaAbook
 *
 * get the path of addressbok. Caller must free the returned value
 *
 * Returns: a gchar*   
 */
gchar* 
rubrica_abook_get_path (RubricaAbook* abook)
{
  g_return_val_if_fail(IS_RUBRICA_ABOOK(abook), NULL);
  
  return g_strdup(abook->private->path);   
}



/**
 * rubrica_abook_get_selected_card_id
 * @abook: a #RubricaAbook
 *
 * get the id of the active card. Caller must free the returned value
 * (DEPRECATED)
 *
 * Returns: a gchar* 
 */
gchar*        
rubrica_abook_get_selected_card_id (RubricaAbook* abook)
{
  g_return_val_if_fail(IS_RUBRICA_ABOOK(abook), NULL);
  
  return g_strdup(abook->private->id);
}



/**
 * rubrica_abook_get_items
 * @abook: a #RubricaAbook
 *
 * get the number of cards into addressbok
 *
 * Returns: integer 
 */
gint
rubrica_abook_get_items (RubricaAbook* abook)
{
  g_return_val_if_fail(IS_RUBRICA_ABOOK(abook), 0);

  return g_list_length(abook->private->cards);
}



/**
 * rubrica_abook_add_card
 * @abook: a #RubricaAbook
 * @card: a #RubricaPersonalCard or #RubricaCompanyCard object
 *
 * append the given card into the addressbook
 */
void   
rubrica_abook_add_card (RubricaAbook* abook, gpointer card)
{
  g_return_if_fail(IS_RUBRICA_ABOOK(abook));
 
  abook->private->cards = g_list_append(abook->private->cards, card);
  abook->private->iter  = abook->private->cards;

  g_signal_emit_by_name(abook, "addressbook_changed", NULL, G_TYPE_NONE);
  g_signal_emit_by_name(abook, "card_added", card, G_TYPE_POINTER);
}



/**
 * rubrica_abook_remove_card
 * @abook: a #RubricaAbook
 * @card: a #RubricaPersonalCard or #RubricaCompanyCard object
 *
 * remove the given card from addressbook 
 */
void   
rubrica_abook_remove_card (RubricaAbook* abook, gpointer card)
{
  g_return_if_fail(IS_RUBRICA_ABOOK(abook));
  g_return_if_fail(card != NULL);

  g_object_set(card, "deleted", TRUE, NULL);
  abook->private->cards = g_list_remove(abook->private->cards, card);
  abook->private->iter  = abook->private->cards;
  abook->private->trash = g_list_append(abook->private->trash, card);
  
  g_signal_emit_by_name(abook, "card_removed", card, G_TYPE_POINTER);
  g_signal_emit_by_name(abook, "addressbook_changed", NULL, G_TYPE_NONE);
}



/**
 * rubrica_abook_replace_card
 * @abook: a #RubricaAbook
 * @old: a #RubricaPersonalCard or #RubricaCompanyCard object
 * @new: a #RubricaPersonalCard or #RubricaCompanyCard object
 * 
 * replace the old card with the new one
 */
void   
rubrica_abook_replace_card (RubricaAbook* abook, gpointer old, gpointer new)
{
  gint pos;
  GList* node = NULL;
  gpointer card = NULL;

  g_return_if_fail(IS_RUBRICA_ABOOK(abook));
      
  pos  = g_list_index(abook->private->cards, old);
  node = g_list_nth(abook->private->cards, pos);
  if (node)
    {
      abook->private->cards = g_list_remove_link(abook->private->cards, node);
      
      card = node->data;
      rubrica_card_free(RUBRICA_CARD(card));
      
      g_list_free_1(node);

      abook->private->cards = g_list_insert(abook->private->cards, new, pos);
      
      g_signal_emit_by_name(abook, "addressbook_changed", NULL, G_TYPE_NONE);
      g_signal_emit_by_name(abook, "card_replaced", new, G_TYPE_POINTER);
    }
}



/**
 * rubrica_abook_get_card
 * @abook: a #RubricaAbook
 *
 * Get the first card in addressbook. 
 *
 * Returns: a pointer to the card or NULL if addressbook is empty
 */
gpointer 
rubrica_abook_get_card (RubricaAbook* abook)
{
  g_return_val_if_fail(IS_RUBRICA_ABOOK(abook), NULL);

  if (abook->private->iter)
    return abook->private->iter->data;

  return NULL;
}


gpointer      
rubrica_abook_get_selected_card (RubricaAbook* abook)
{
  gchar* id;
  gpointer card;
  
  g_return_val_if_fail(IS_RUBRICA_ABOOK(abook), NULL);

  id = rubrica_abook_get_selected_card_id(abook);
  
  card = rubrica_abook_get_card_by_id(abook, id);
  g_free(id);

  return card;
}


/**
 * rubrica_abook_get_card_by_id
 * @abook: a #RubricaAbook
 * @id: the card id
 * 
 * get the card with "id" id
 *
 * Returns: a pointer to the card or NULL if addressbook doesn't own that card
 */
gpointer      
rubrica_abook_get_card_by_id (RubricaAbook* abook, gchar* id)
{
  gpointer card = NULL;

  g_return_val_if_fail(IS_RUBRICA_ABOOK(abook), NULL);
  g_return_val_if_fail(id != NULL, NULL);
  
  rubrica_abook_reset_book(abook);
  for (card = rubrica_abook_get_card(abook); card; 
       card = rubrica_abook_get_next_card(abook))
    {
      gchar* card_id;
      
      card_id = rubrica_card_get_id(RUBRICA_CARD(card));
      if (g_ascii_strcasecmp(id, card_id) == 0)
	return card;      
    }  

  return NULL;
}


/**
 * rubrica_abook_find_cards_in_group
 * @abook: a #RubricaAbook
 * @group_name: the name of the group
 *
 * Find all cards that belongs to the given group. 
 * Found cards's ID has inserted in a GList. Caller must free returned GList
 * 
 * returns: a GList*
 */
GList*
rubrica_abook_find_cards_in_group (RubricaAbook* abook, gchar* group_name)
{
  gpointer data;
  GList* found = NULL;

  g_return_val_if_fail(IS_RUBRICA_ABOOK(abook), NULL);
  g_return_val_if_fail(group_name != NULL, NULL);

  rubrica_abook_reset_book(abook);
  data = rubrica_abook_get_card(abook);
  for (; data; data = rubrica_abook_get_next_card(abook))
    {
      gchar* id;

      if (rubrica_card_belong_to_group(RUBRICA_CARD(data), group_name))
	{
	  id = g_strdup(rubrica_card_get_id(RUBRICA_CARD(data)));
	  
	  found = g_list_append(found, id);
	}
    }
  
  return found;
}



/**
 * rubrica_abook_reset_book
 * @abook: a #RubricaAbook
 * 
 * reset the private addressbook iterator. The iterator 
 * will point the head of cards's list.
 */
void     
rubrica_abook_reset_book (RubricaAbook* abook)
{
  g_return_if_fail(IS_RUBRICA_ABOOK(abook));
 
  abook->private->iter = abook->private->cards;
}


/**
 * rubrica_abook_get_next_card
 * @abook: a #RubricaAbook
 *
 * get the next card in addressbook
 * 
 * Returns: a pointer to the card or NULL if the cards's end list is reached.
 */
gpointer
rubrica_abook_get_next_card (RubricaAbook* abook)
{
  g_return_val_if_fail(IS_RUBRICA_ABOOK(abook), NULL);
 
  abook->private->iter = g_list_next(abook->private->iter);
  if (abook->private->iter)
    return abook->private->iter->data;
  else
    abook->private->iter = g_list_last(abook->private->cards);  

  return NULL;
}


/**
 * rubrica_abook_get_prev_card
 * @abook: a #RubricaAbook
 *
 * get the previous card in addressbook
 * 
 * Returns: a pointer to the card or NULL if the cards's head list is reached.
 */
gpointer 
rubrica_abook_get_prev_card (RubricaAbook* abook)
{
  g_return_val_if_fail(IS_RUBRICA_ABOOK(abook), NULL);
 
  abook->private->iter = g_list_previous(abook->private->iter);
  if (abook->private->iter)
    return abook->private->iter->data;
  else 
    abook->private->iter = abook->private->cards;

  return NULL;
}
