/*
*  RAL -- Rubrica Addressbook Library
*  file: address.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <time.h>
#include <glib.h>
#include <glib-object.h>

#include "address.h"
#include "utils.h"


enum { 
  ADDRESS_TYPE = 1, 
  ADDRESS_STREET, 
  ADDRESS_NUMBER, 
  ADDRESS_CITY, 
  ADDRESS_ZIP, 
  ADDRESS_PROVINCE, 
  ADDRESS_STATE, 
  ADDRESS_COUNTRY 
}; 


struct _RubricaAddressPrivate {
  gint   type;
  gchar* street;
  gchar* number;
  gchar* city;
  gchar* zip;
  gchar* province;
  gchar* state;
  gchar* country;

  gboolean dispose_has_run;
};


typedef struct {
  gint   token;
  gchar* label;
} AddressConversion;


static AddressConversion table[] = {
  {RUBRICA_ADDRESS_PREF,    "pref"   },
  {RUBRICA_ADDRESS_HOME,    "home"   },
  {RUBRICA_ADDRESS_WORK,    "work"   },
  {RUBRICA_ADDRESS_PARCEL,  "parcel" },
  {RUBRICA_ADDRESS_POSTAL,  "postal" },
  {RUBRICA_ADDRESS_INTL,    "intl"   },
  {RUBRICA_ADDRESS_DOM,     "dom"    },
  {RUBRICA_ADDRESS_COMPANY, "company"},
  {RUBRICA_ADDRESS_UNKNOWN, "unknown"},
  {RUBRICA_ADDRESS_INVALID, "invalid"},
};


static void rubrica_address_class_init   (RubricaAddressClass* klass);
static void rubrica_address_init         (RubricaAddress* obj);

static void rubrica_address_dispose      (RubricaAddress* obj);
static void rubrica_address_finalize     (RubricaAddress* obj);

static void rubrica_address_set_property (GObject* obj, guint property_id,
					  const GValue* value, 
					  GParamSpec* spec);
static void rubrica_address_get_property (GObject* obj, guint property_id,
					  GValue* value, GParamSpec* spec);


GType
rubrica_address_get_type()
{
  static GType address_type = 0;
  
  if (!address_type)
    {
      static const GTypeInfo address_info =
	{
	  sizeof(RubricaAddressClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_address_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaAddress),
	  0,
	  (GInstanceInitFunc) rubrica_address_init
	};

      address_type = g_type_register_static (G_TYPE_OBJECT, 
					     "RubricaAddress",
					     &address_info, 0);
    }
  
  return address_type;
}


static void
rubrica_address_class_init(RubricaAddressClass* klass)
{
  GObjectClass *class;
  GParamSpec* pspec;
  
  class  = G_OBJECT_CLASS (klass);

  class->dispose  = (GObjectFinalizeFunc) rubrica_address_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_address_finalize;

  class->set_property = rubrica_address_set_property;
  class->get_property = rubrica_address_get_property;

  /* class property
   */
  
  /**
   * RubricaAddress:address-type:
   *
   * defines the address's type. See #RubricaAddressType for valid values
   */
  pspec = g_param_spec_int("address-type",  
			   "address's type",  
			   "store the address's type, i.e. " 
			   "RUBRICA_ADDRESS_PREF or RUBRICA_ADDRESS_WORK",  
			   RUBRICA_ADDRESS_PREF,         
			   RUBRICA_ADDRESS_INVALID,    
			   RUBRICA_ADDRESS_UNKNOWN,    
			   G_PARAM_READWRITE);  
  g_object_class_install_property(class, ADDRESS_TYPE, pspec);     

  
  /**
   * RubricaAddress:street:
   *
   * the address's street
   */
  pspec = g_param_spec_string("street", 
			      "address's street", 
			      "the address's street",  
			      NULL,    
			     G_PARAM_READWRITE); 
  g_object_class_install_property(class, ADDRESS_STREET, pspec);    

  /**
   * RubricaAddress:address-number:
   *
   * the street number
   */
  pspec = g_param_spec_string("street-number", 
			      "the street number", 
			      "the street number",  
			      NULL,    
			      G_PARAM_READWRITE); 
  g_object_class_install_property(class, ADDRESS_NUMBER, pspec);    

  /**
   * RubricaAddress:city:
   *
   * the address's city
   */
  pspec = g_param_spec_string("city", 
			      "the address's city", 
			      "the address's city",  
			      NULL,    
			       G_PARAM_READWRITE); 
  g_object_class_install_property(class, ADDRESS_CITY, pspec);    

  /**
   * RubricaAddress:zip:
   *
   * the postal code
   */
  pspec = g_param_spec_string("zip", 
			      "postal code", 
			      "the postal code",  
			      NULL,    
			      G_PARAM_READWRITE); 
  g_object_class_install_property(class, ADDRESS_ZIP, pspec);    

  /**
   * RubricaAddress:province:
   *
   * the address's province (example: Rm (Rome), Mi (Milan) , Na (Naples) ...) 
   */
  pspec = g_param_spec_string("province", 
			      "province", 
			      "the address's province",  
			      NULL,    
			      G_PARAM_READWRITE); 
  g_object_class_install_property(class, ADDRESS_PROVINCE, pspec);    

  /**
   * RubricaAddress:state:
   *
   * the address's state (examples: UE, USA, ...)
   */
  pspec = g_param_spec_string("state", 
			      "the state", 
			      "the state",  
			      NULL,    
			     G_PARAM_READWRITE); 
  g_object_class_install_property(class, ADDRESS_STATE, pspec);   
 
  /**
   * RubricaAddress:country:
   *
   * the address's country (example: Italy, France, ...)
   */
  pspec = g_param_spec_string("country", 
			      "the country", 
			      "the country",  
			      NULL,    
			      G_PARAM_READWRITE); 
  g_object_class_install_property(class, ADDRESS_COUNTRY, pspec); 
  
}


static void
rubrica_address_init(RubricaAddress* self)
{
  self->private = g_new(RubricaAddressPrivate, 1);
  self->private->dispose_has_run = FALSE;

  self->private->street   = NULL;
  self->private->number   = NULL;
  self->private->city     = NULL;
  self->private->zip      = NULL;
  self->private->province = NULL;
  self->private->state    = NULL;
  self->private->country  = NULL;  
}



static void 
rubrica_address_dispose (RubricaAddress* self)
{
  g_return_if_fail(IS_RUBRICA_ADDRESS(self));
  
  if (self->private->dispose_has_run)
    return;

  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_address_finalize (RubricaAddress* self)
{
  g_return_if_fail(IS_RUBRICA_ADDRESS(self));

  rubrica_utils_free_string(self->private->street);
  rubrica_utils_free_string(self->private->number);
  rubrica_utils_free_string(self->private->city);
  rubrica_utils_free_string(self->private->zip);
  rubrica_utils_free_string(self->private->province);
  rubrica_utils_free_string(self->private->state);
  rubrica_utils_free_string(self->private->country);

  g_free(self->private);
  self->private = NULL;
}


static void 
rubrica_address_set_property (GObject* obj, guint property_id,
			      const GValue* value, GParamSpec* spec)
{
  RubricaAddress* self = (RubricaAddress*) obj;
  
  switch (property_id) 
    {
    case ADDRESS_TYPE:
      self->private->type = g_value_get_int(value);
      break;

    case ADDRESS_STREET:
      g_free(self->private->street);
      self->private->street = g_value_dup_string(value);
      break;

    case ADDRESS_NUMBER:
      g_free(self->private->number);
      self->private->number = g_value_dup_string(value);
      break;

    case ADDRESS_CITY:
      g_free(self->private->city);
      self->private->city = g_value_dup_string(value);
      break;

    case ADDRESS_ZIP:
      g_free(self->private->zip);
      self->private->zip = g_value_dup_string(value);
      break;

    case ADDRESS_PROVINCE:
      g_free(self->private->province);
      self->private->province = g_value_dup_string(value);
      break;

    case ADDRESS_STATE:
      g_free(self->private->state);
      self->private->state = g_value_dup_string(value);
      break;

    case ADDRESS_COUNTRY:
      g_free(self->private->country);
      self->private->country = g_value_dup_string(value);
      break;
      
    default:
      break;  
    }   
}


static void 
rubrica_address_get_property (GObject* obj, guint property_id,
			      GValue* value, GParamSpec* spec)
{
  RubricaAddress* self = (RubricaAddress*) obj;
  
  switch (property_id) 
    {
    case ADDRESS_TYPE:
      g_value_set_int(value, self->private->type);
      break; 

    case ADDRESS_STREET:
      g_value_set_string(value, self->private->street);
      break; 

    case ADDRESS_NUMBER:
      g_value_set_string(value, self->private->number);
      break; 

    case ADDRESS_CITY:
      g_value_set_string(value, self->private->city);
      break; 

    case ADDRESS_ZIP:
      g_value_set_string(value, self->private->zip);
      break; 

    case ADDRESS_PROVINCE:
      g_value_set_string(value, self->private->province);
      break; 

    case ADDRESS_STATE:
      g_value_set_string(value, self->private->state);
      break; 

    case ADDRESS_COUNTRY:
      g_value_set_string(value, self->private->country);
      break; 
      
    default:
      break;  
    }  
}




/* **************** Public
 */


/**
 * rubrica_address_new
 *
 * create a new #RubricaAddress
 * 
 * Returns: a #RubricaAddress*
 */
RubricaAddress*
rubrica_address_new(void)
{
  RubricaAddress* address;

  address = g_object_new(rubrica_address_get_type(), NULL);

  return address;
}


/**
 * rubrica_address_free
 * @address: a #RubricaAddress
 *
 * free the #RubricaAddress*
 */
void
rubrica_address_free(RubricaAddress* address)
{
  g_return_if_fail(IS_RUBRICA_ADDRESS(address));

  g_object_unref(address);
}

/**
 * rubrica_address_decode_type
 * @type: a #RubricaAddressType
 *
 * decode the address type
 * 
 * Returns: a gchar*
 */
gchar* 
rubrica_address_decode_type (RubricaAddressType type)
{
  if ((type < RUBRICA_ADDRESS_PREF) || (type > RUBRICA_ADDRESS_UNKNOWN))
    return "unknown";

  return table[type].label;    
}


/**
 * rubrica_address_encode_type
 * @type: a gchar*
 *
 * encode the address type
 * 
 * Returns: a RubricaAddressType value or RUBRICA_ADDRESS_UNKNOWN
 */
RubricaAddressType 
rubrica_address_encode_type (gchar* type)
{
  RubricaAddressType ret = RUBRICA_ADDRESS_INVALID;

  if (!type) return ret;
  
  if (g_ascii_strcasecmp(type, "pref") == 0)
    ret = RUBRICA_ADDRESS_PREF;
  else if (g_ascii_strcasecmp(type, "home") == 0)
    ret = RUBRICA_ADDRESS_HOME;
  else if (g_ascii_strcasecmp(type, "work") == 0)
    ret = RUBRICA_ADDRESS_WORK;
  else if (g_ascii_strcasecmp(type, "parcel") == 0)
    ret = RUBRICA_ADDRESS_PARCEL;
  else if (g_ascii_strcasecmp(type, "postal") == 0)
    ret = RUBRICA_ADDRESS_POSTAL;
  else if (g_ascii_strcasecmp(type, "intl") == 0)
    ret = RUBRICA_ADDRESS_INTL;
  else if (g_ascii_strcasecmp(type, "dom") == 0)
    ret = RUBRICA_ADDRESS_DOM;
  else if (g_ascii_strcasecmp(type, "company") == 0)
    ret = RUBRICA_ADDRESS_COMPANY;
  else
    ret = RUBRICA_ADDRESS_UNKNOWN;
 
  return ret;    
}



/**
 * rubrica_address_copy
 * @address: a #RubricaAddress
 *
 * copy the given #RubricaAddress
 * 
 * Returns: a new allocated #RubricaAddress
 */
RubricaAddress* 
rubrica_address_copy  (RubricaAddress* address)
{
  RubricaAddress* new;
  RubricaAddressType type;
  gchar *street, *number, *city;
  gchar *zip, *province, *state, *country;
  
  g_return_val_if_fail(IS_RUBRICA_ADDRESS(address), NULL);
  
  new = rubrica_address_new();

  g_object_get(G_OBJECT(address), "address-type", &type, "street", &street,
	       "street-number", &number, "city", &city, "zip", &zip,
	       "province", &province, "state", &state, 
	       "country", &country, NULL);

  g_object_set(G_OBJECT(new), "address-type", type, "street", street,
	       "street-number", number, "city", city, "zip", zip,
	       "province", province, "state", state, 
	       "country", country, NULL);
  
  return new;
}



