/*
*  RAL -- Rubrica Addressbook Library
*  file: card.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <time.h>
#include <string.h>
#include <glib.h>
#include <glib-object.h>
#include <libxml/tree.h>

#include "card.h"
#include "group.h"
#include "utils.h"


gulong personal_id = 0;
gulong company_id  = 0;


/*  property enumeration
*/
enum {
  CARD_NAME = 1,
  CARD_TYPE,
  CARD_RANK,
  CARD_DELETABLE,
  CARD_DELETED,
  CARD_MARKED
};
   

struct _RubricaCardPrivate {
  gchar*           id;           /* card's unique id                 */

  gchar*           name;         /* card's name                      */
  gchar*           type;         /* card's type                      */
  RubricaRank      rank;         /* card's rank                      */
     
  gboolean         deletable;    /* true if card is marked deletable */
  gboolean         deleted;      /* is this a deleted card ?         */
  gboolean         marked;       /* true if card was marked          */

  RubricaGroupSet* set;          /* card's groups set                */

  GList*           addresses;
  GList*           nets;         /* web, email and irc uries         */
  GList*           phones;


  time_t           created;      /* card's creation time             */
  time_t           changed;      /* card's last change time          */

  GList*           grp_iter;     /* groups, addresses, telephones,   */
  GList*           addr_iter;    /* nets iterators                   */
  GList*           net_iter;
  GList*           phone_iter;

  gboolean         dispose_has_run;
};


static void rubrica_card_class_init   (RubricaCardClass* klass);
static void rubrica_card_init         (RubricaCard* self);

static void rubrica_card_dispose      (RubricaCard* self);
static void rubrica_card_finalize     (RubricaCard* self);

static void rubrica_card_set_property (GObject* obj, guint property_id,
				       const GValue* value, GParamSpec* spec);
static void rubrica_card_get_property (GObject* obj, guint property_id,
				       GValue* value, GParamSpec* spec);


static gulong rubrica_card_generate_id (RubricaCard* card, gchar* type);
static void   rubrica_card_align_id    (RubricaCard* card, gchar* id, 
					gchar* type);
static void   rubrica_card_real_copy   (RubricaCard* new, RubricaCard* old);


/*  Private functions
*/
GType
rubrica_card_get_type()
{
  static GType rubrica_card_type = 0;
  
  if (!rubrica_card_type)
    {
      static const GTypeInfo rubrica_card_info =
	{
	  sizeof(RubricaCardClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_card_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaCard),
	  0,
	  (GInstanceInitFunc) rubrica_card_init
	};

      rubrica_card_type = g_type_register_static (G_TYPE_OBJECT, "RubricaCard",
						  &rubrica_card_info, 0);
    }

  return rubrica_card_type;
}


static void
rubrica_card_class_init(RubricaCardClass* klass)
{
  GObjectClass *class;
  GParamSpec* pspec;
  
  klass->free            = NULL;      /* free method, child will set */
  klass->generate_id     = (gpointer) rubrica_card_generate_id;
  klass->align_id        = (gpointer) rubrica_card_align_id;

  klass->write_infos     = NULL;      /* write card's infos            */
  klass->write_contact   = NULL;      /* write personal/company's data */
  klass->write_group     = NULL;      /* write group's data            */ 
  klass->write_work      = NULL;      /* write work's data             */
  klass->write_addresses = NULL;      /* write addresses's data        */
  klass->write_net       = NULL;      /* write net's data              */
  klass->write_telephone = NULL;      /* write telephone's data        */
  klass->write_notes     = NULL;      /* write notes's data            */

  klass->read_infos      = NULL;      /* read card's infos             */
  klass->read_contact    = NULL;      /* read personal/company's data  */
  klass->read_group      = NULL;      /* read group's data             */ 
  klass->read_work       = NULL;      /* read work's data              */
  klass->read_addresses  = NULL;      /* read addresses's data         */
  klass->read_net        = NULL;      /* read net's data               */
  klass->read_telephone  = NULL;      /* read telephone's data         */
  klass->read_notes      = NULL;      /* read notes's data             */
  
  klass->copy            = NULL;      /* copy a card */

  klass->display         = NULL;      /* show card's data              */

  klass->get_infos       = NULL;      /* return infos on card record   */
  klass->print_card      = NULL;      // unused


  class = G_OBJECT_CLASS (klass);
  class->dispose  = (GObjectFinalizeFunc) rubrica_card_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_card_finalize;

  class->set_property = rubrica_card_set_property;
  class->get_property = rubrica_card_get_property;

  /**
   * RubricaCard:card-name:
   *
   * card's name
   */
  pspec = g_param_spec_string("card-name",
			      "card's name",
			      "the name of the card",
			      NULL,
			      G_PARAM_READWRITE);
  g_object_class_install_property(class, CARD_NAME, pspec);     

  
  /**
   * RubricaCard:card-type:
   *
   * card's type
   */
   pspec = g_param_spec_string("card-type",
			      "card's type",
			      "card's type, must be \"personal\"" 
			      "or \"company\"",
			      NULL,
			      G_PARAM_READABLE);
  g_object_class_install_property(class, CARD_TYPE, pspec);     

  /**
   * RubricaCard:rank:
   *
   * card's rank
   */
   pspec = g_param_spec_int("rank",
			   "card's rank",
			   "the rank user wants for the card",
			   RUBRICA_RANK_NONE,
			   RUBRICA_RANK_VERY_HIGHT,
			   RUBRICA_RANK_NORMAL,
			   G_PARAM_READWRITE);
  g_object_class_install_property(class, CARD_RANK, pspec);     

  /**
   * RubricaCard:deletable:
   *
   * is card deletable?
   */
   pspec = g_param_spec_boolean("deletable",
			       "deletable",
			       "is the card deletable",
			       TRUE,
			       G_PARAM_READWRITE);
  g_object_class_install_property(class, CARD_DELETABLE, pspec);     
  
  /**
   * RubricaCard:deleted:
   *
   * is card deleted?
   */
   pspec = g_param_spec_boolean("deleted",
			       "deleted",
			       "is the card deleted",
			       FALSE,
			       G_PARAM_READWRITE);
  g_object_class_install_property(class, CARD_DELETED, pspec);     
  
  /**
   * RubricaCard:marked:
   *
   * is card marked?
   */
   pspec = g_param_spec_boolean("marked",
			       "marked",
			       "is the card marked",
			       FALSE,
			       G_PARAM_READWRITE);
  g_object_class_install_property(class, CARD_MARKED, pspec);     
}


static void
rubrica_card_init(RubricaCard* self)
{
  self->private = g_new(RubricaCardPrivate, 1);

  self->private->id              = NULL;

  self->private->name            = NULL;
  self->private->type            = NULL;
  self->private->rank            = RUBRICA_RANK_NORMAL;

  self->private->set             = rubrica_group_set_new();

  self->private->deletable       = TRUE;
  self->private->deleted         = FALSE;
  self->private->marked          = FALSE;

  self->private->addresses       = NULL;
  self->private->nets            = NULL;
  self->private->phones          = NULL;

  self->private->created         = time(NULL);
  self->private->changed         = time(NULL);

  self->private->grp_iter        = NULL;
  self->private->addr_iter       = NULL;
  self->private->net_iter        = NULL;
  self->private->phone_iter      = NULL;

  self->private->dispose_has_run = FALSE;
}



static void 
rubrica_card_dispose (RubricaCard* self)
{
  GList* del;  

  g_return_if_fail(IS_RUBRICA_CARD(self));    

  if (self->private->dispose_has_run)
    return;

  self->private->dispose_has_run = TRUE;

  /* free card's children 
  */
  RUBRICA_CARD_CLASS(RUBRICA_CARD_GET_CLASS(self))->free(self);

  /* free card's data
   */
  rubrica_utils_free_string(self->private->id);
  rubrica_utils_free_string(self->private->name);
  rubrica_utils_free_string(self->private->type);
  self->private->id   = NULL;
  self->private->name = NULL;
  self->private->type = NULL;

  /* groups */
  rubrica_group_set_free(RUBRICA_GROUP_SET(self->private->set));

  /* addresses */
  del = self->private->addresses;
  for (; del; del = del->next)
    {
      RubricaAddress* address = del->data;;

      if(address)
	rubrica_address_free(RUBRICA_ADDRESS(address));
    }
  self->private->addresses = NULL;

  /* net addresses */
  del = self->private->nets;
  for (; del; del = del->next)
    {
      RubricaNetAddress* net = del->data;;

      if(net)
	rubrica_net_address_free(RUBRICA_NET_ADDRESS(net));
    }
  self->private->nets = NULL;

  /* telephone numbers */
  del = self->private->phones;
  for (; del; del = del->next)
    {
      RubricaTelephone* tel = del->data;;

      if(tel)
	rubrica_telephone_free(RUBRICA_TELEPHONE(tel));
    }
  self->private->phones = NULL;   

  self->private->grp_iter   = NULL;
  self->private->addr_iter  = NULL;
  self->private->net_iter   = NULL;
  self->private->phone_iter = NULL;
}


static void 
rubrica_card_finalize (RubricaCard* self)
{
  g_return_if_fail(IS_RUBRICA_CARD(self));

  g_free(self->private);
  self->private = NULL;
}


static void 
rubrica_card_set_property (GObject* obj, guint property_id,
			   const GValue* value, GParamSpec* spec)
{
  RubricaCard* self = (RubricaCard*) obj;

  switch(property_id)
    {
    case CARD_NAME:
      g_free(self->private->name);
      self->private->name = g_value_dup_string(value);
      break;

    case CARD_RANK:
      self->private->rank = g_value_get_int(value);
      break;

    case CARD_DELETABLE:
      self->private->deletable = g_value_get_boolean(value);
      break;

    case CARD_DELETED:
      self->private->deleted = g_value_get_boolean(value);
      break;

    case CARD_MARKED:
      self->private->marked = g_value_get_boolean(value);
      break;
      
    default:
      break;
    }
}

static void 
rubrica_card_get_property (GObject* obj, guint property_id,
			   GValue* value, GParamSpec* spec)
{
  RubricaCard* self = (RubricaCard*) obj;

  switch(property_id)
    {
    case CARD_NAME:
      g_value_set_string(value, self->private->name);
      break;

    case CARD_TYPE:
      g_value_set_string(value, self->private->type);   
      break;

    case CARD_RANK:
      g_value_set_int(value, self->private->rank);
      break;

    case CARD_DELETABLE:
      g_value_set_boolean(value, self->private->deletable);
      break;

    case CARD_DELETED:
      g_value_set_boolean(value, self->private->deleted);
      break;

    case CARD_MARKED:
      g_value_set_boolean(value, self->private->marked);
      break;
      
    default:
      break;
    }
}




/*
 * rubrica_card_generate_id
 * private method, create the new card's id  
 */
static gulong 
rubrica_card_generate_id (RubricaCard* self, gchar* type) 
{
  gulong ret = 0L;

  g_return_val_if_fail(IS_RUBRICA_CARD(self), 0L); 
  
  if (g_ascii_strcasecmp(type, "personal") == 0)
    {
      self->private->type = g_strdup("personal");
      ret = ++personal_id;
    }
  else if (g_ascii_strcasecmp(type, "company") == 0)
    {
      self->private->type = g_strdup("company");
      ret = ++company_id;
    }

  return ret;
}


/*
 * rubrica_card_align_id
 * private method, align the object's private id with the last read 
 */
static void   
rubrica_card_align_id (RubricaCard* self, gchar* id, gchar* type)
{
  gulong cid = 0L;  

  g_return_if_fail(IS_RUBRICA_CARD(self)); 
  g_return_if_fail(id != NULL);
  
  id+=2;       /* sposto il ptr avanti di due byte (pcxxx) */  
  cid = atol(id);
   
  if (g_ascii_strcasecmp(type, "personal") == 0)
    {
      if (cid >= personal_id)
	personal_id = cid;
    }
  else if (g_ascii_strcasecmp(type, "company") == 0)
    {
      if (cid >= company_id)
	company_id = cid;
    }  
}




/*
 * rubrica_card_real_copy
 * private method. Copy from the old card in new card the 
 * RubricaPersonalCard and RubricaCompanycard common values, it est:
 * card infos, groups, addresses, net addresses, telephone numbers
 */
static void
rubrica_card_real_copy (RubricaCard* new, RubricaCard* old)
{
  RubricaGroup* group;
  RubricaAddress* address;
  RubricaNetAddress* net;
  RubricaTelephone* tel;
  gchar *name;
  gint rank;
  gboolean deletable, deleted, marked;
  time_t time;

  g_return_if_fail(IS_RUBRICA_CARD(new));
  g_return_if_fail(IS_RUBRICA_CARD(old));
  
  g_object_get(G_OBJECT(old), "card-name", &name, "rank", &rank,
	       "deletable", &deletable, "deleted", &deleted, 
	       "marked", &marked, NULL);

  g_object_set(G_OBJECT(new), "card-name", name, "rank", rank,
	       "deletable", deletable, "deleted", deleted, 
	       "marked", marked, NULL);

  time = rubrica_card_get_creation_time(old);
  rubrica_card_set_creation_time(new, time);

  time = rubrica_card_get_modification_time(old);
  rubrica_card_set_modification_time(new, time);
  
  group = rubrica_card_get_group(RUBRICA_CARD(old));
  for  (; group; group = rubrica_card_get_next_group(RUBRICA_CARD(old)))
    {
      RubricaGroup* grp;
      
      grp = rubrica_group_copy(RUBRICA_GROUP(group));
      if (grp)
	rubrica_card_add_group(RUBRICA_CARD(new), grp);
    }
  
  address = rubrica_card_get_address(RUBRICA_CARD(old));
  for  (; address; address = rubrica_card_get_next_address(RUBRICA_CARD(old)))
    {
      RubricaAddress* add;
      
      add = rubrica_address_copy(RUBRICA_ADDRESS(address));
      if (add)
	rubrica_card_add_address(RUBRICA_CARD(new), add);
    }
 
  net = rubrica_card_get_net_address(RUBRICA_CARD(old));
  for (; net; net = rubrica_card_get_next_net_address(RUBRICA_CARD(old)))
    {
      RubricaNetAddress* nt;

      nt = rubrica_net_address_copy(RUBRICA_NET_ADDRESS(net));
      if (nt)
	rubrica_card_add_net_address(RUBRICA_CARD(new), nt);
    }
  
  tel = rubrica_card_get_telephone(RUBRICA_CARD(old));
  for (; tel; tel = rubrica_card_get_next_telephone(RUBRICA_CARD(old)))
    {
      RubricaTelephone* tn;

      tn = rubrica_telephone_copy(RUBRICA_TELEPHONE(tel));
      if (tn)
	rubrica_card_add_telephone(RUBRICA_CARD(new), tn);
    }
}



/* ************************************  Public Functions
 */

/**
 * rubrica_card_new
 *
 * create a new #RubricaCard
 *
 * Returns: a new allocated #RubricaCard*
 */
RubricaCard*  
rubrica_card_new(void)
{
  RubricaCard* card;

  card = g_object_new(rubrica_card_get_type(), NULL);

  return card;
}


/**
 * rubrica_card_free
 * @card: a #RubricaCard
 * 
 * free the #RubricaCard*
 */
void 
rubrica_card_free(RubricaCard* card)
{
  g_return_if_fail(IS_RUBRICA_CARD(card));
  
  g_object_unref(G_OBJECT(card));  
}


/**
 * rubrica_card_copy
 * @card: a #RubricaCard
 *
 * copy the given #RubricaCard*
 *
 * Returns: a new allocated copy of the card
 */
RubricaCard*
rubrica_card_copy (RubricaCard* card)
{
  RubricaCardClass* klass;
  RubricaCard* new;
  
  g_return_val_if_fail(IS_RUBRICA_CARD(card), NULL);   

  klass = RUBRICA_CARD_GET_CLASS(card);

  if (RUBRICA_CARD_CLASS(klass)->copy)
    {
      new = RUBRICA_CARD_CLASS(klass)->copy(RUBRICA_CARD(card));    
      rubrica_card_real_copy(RUBRICA_CARD(new), RUBRICA_CARD(card));

      return (gpointer) new;
    }
  
  return NULL;
}


/**
 * rubrica_card_id_align
 * @card:
 * @id:
 * @card_type:
 *
 * align the card's id. User shouldn't use this function.
 */
void 
rubrica_card_id_align (RubricaCard* card, gchar* id, gchar* card_type)
{
  RubricaCardClass* klass;
  
  g_return_if_fail(IS_RUBRICA_CARD(card));   
  g_return_if_fail(id != NULL);
  g_return_if_fail(card_type != NULL);

  klass = RUBRICA_CARD_GET_CLASS(card);

  RUBRICA_CARD_CLASS(klass)->align_id(card, id, card_type);  
}


/**
 * rubrica_card_set_id
 * @card: a #RubricaCard
 * @id: an unique id
 *
 * set the card's id.
 */
void      
rubrica_card_set_id (RubricaCard* card, gchar* id) 
{ 
  g_return_if_fail(IS_RUBRICA_CARD(card));   
  
  rubrica_utils_free_string(card->private->id); 
  card->private->id = g_strdup(id); 
}


/**
 * rubrica_card_set_creation_time
 * @card: a #RubricaCard
 * @time: a time_t value
 * 
 * set the card's creation time.
 */
void  
rubrica_card_set_creation_time (RubricaCard* card, time_t time) 
{ 
  g_return_if_fail(IS_RUBRICA_CARD(card)); 

  card->private->created = time; 
}



/**
 * rubrica_card_set_modification_time
 * @card: a #RubricaCard
 * @time: a time_t value
 *
 * set card's last modification time.
 */
void 
rubrica_card_set_modification_time (RubricaCard* card, time_t time)
{
  g_return_if_fail(IS_RUBRICA_CARD(card));

  card->private->changed = time;
}


/**
 * rubrica_card_get_id
 * @card: a #RubricaCard
 *
 * get the card's id. Caller must free the returned value
 * 
 * Returns: a gchar*
 */
gchar*
rubrica_card_get_id (RubricaCard* card)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(card), NULL);

  return g_strdup(card->private->id);
}


/**
 * rubrica_card_get_creation_time
 * @card: a #RubricaCard
 *
 * get the card's creation time
 *
 * Returns: a time_t 
 */
time_t   
rubrica_card_get_creation_time (RubricaCard* card)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(card), -1);

  return card->private->created;
}


/**
 * rubrica_card_get_modification_time 
 * @card: a #RubricaCard
 *
 * get the last time that card was modified
 *
 * Returns: a time_t
 */
time_t   
rubrica_card_get_modification_time (RubricaCard* card)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(card), -1);

  return card->private->changed;
}



RubricaInfos* 
rubrica_card_get_infos (RubricaCard* card)
{
  RubricaCardClass* klass;
  RubricaInfos* infos;
  gpointer address, tel;

  g_return_val_if_fail(IS_RUBRICA_CARD(card), NULL);

  klass = RUBRICA_CARD_GET_CLASS(card);

  if (RUBRICA_CARD_CLASS(klass)->get_infos)
    {
      infos = RUBRICA_CARD_CLASS(klass)->get_infos(RUBRICA_CARD(card));
      
      g_object_get(RUBRICA_CARD(card), "card-name", &infos->label, NULL);

      infos->id    = rubrica_card_get_id(RUBRICA_CARD(card));
      infos->web   = g_strdup(rubrica_card_get_home_page(RUBRICA_CARD(card)));
      infos->email = g_strdup(rubrica_card_get_email(RUBRICA_CARD(card)));
      infos->irc   = g_strdup(rubrica_card_get_irc(RUBRICA_CARD(card)));
 
      address = rubrica_card_get_address(RUBRICA_CARD(card));
      tel     = rubrica_card_get_telephone(RUBRICA_CARD(card));
      
      if (address)
	g_object_get(RUBRICA_ADDRESS(address), 
		     "city", &infos->city, 
		     "country", &infos->country, NULL); 
      else
	infos->city = infos->country = g_strdup(NULL);
      
      
      if (tel)
	g_object_get(RUBRICA_TELEPHONE(tel), 
		     "telephone-number", &infos->telephone, NULL);
      else
	infos->telephone = g_strdup(NULL);

      return infos;     
   }
  
  return NULL;
}


void          
rubrica_card_free_infos (RubricaInfos *infos)
{
  g_return_if_fail(infos != NULL);
  
  g_free(infos->id);
  g_free(infos->label);
  g_free(infos->first);
  g_free(infos->last);
  g_free(infos->prof);
  g_free(infos->city);
  g_free(infos->country);
  g_free(infos->assignment);
  g_free(infos->web);
  g_free(infos->email);
  g_free(infos->irc);
  g_free(infos->telephone);
 
  g_free(infos);
  infos = NULL;
}



/**
 * rubrica_card_add_group
 * @card: a #RubricaCard
 * @group: a #RubricaGroup
 * 
 * add a group to card (card belongs to added group)
 */
void 
rubrica_card_add_group (RubricaCard* card, RubricaGroup* group)
{
  g_return_if_fail(IS_RUBRICA_CARD(card));
  g_return_if_fail(IS_RUBRICA_GROUP(group));

  rubrica_group_set_add_group(card->private->set, group);
}


/**
 * rubrica_card_delete_group
 * @card: a #RubricaCard
 * @name: group's name
 *
 * delete a group from card's groups list (i.e. remove a card from a group)
 *
 * Returns: TRUE if group was successfully deleted, FALSE otherwise
 */
gboolean      
rubrica_card_delete_group (RubricaCard* card, gchar* name)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(card), FALSE);
  g_return_val_if_fail(name != NULL, FALSE);

  return rubrica_group_set_delete_group_by_name(card->private->set, name);
}



/**
 * rubrica_card_rename_group
 * @card: a #RubricaCard
 * @oldname: old group's name
 * @newname: new group's name
 *
 * rename the old name group with the new name group
 *
 * Returns: TRUE if replaced, FALSE otherwise
 */
gboolean    
rubrica_card_rename_group (RubricaCard* card, gchar* oldname, gchar* newname)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(card), FALSE); 
  g_return_val_if_fail(oldname != NULL, FALSE);
  g_return_val_if_fail(newname != NULL, FALSE);

  return rubrica_group_set_rename_group(card->private->set, oldname, newname);
}


/**
 * rubrica_card_add_address
 * @card: a #RubricaCard
 * @address: a #RubricaAddress
 * 
 * add an address to card
 */
void 
rubrica_card_add_address (RubricaCard* card, RubricaAddress* address)
{
  g_return_if_fail(IS_RUBRICA_CARD(card));
  g_return_if_fail(IS_RUBRICA_ADDRESS(address));

  card->private->addresses = g_list_append(card->private->addresses, address);

  /* init address iterator */
  if (!card->private->addr_iter)
    card->private->addr_iter = card->private->addresses;
}



/**
 * rubrica_card_delete_address
 * @card: a #RubricaCard
 * @address: a #RubricaAddress
 * 
 * delete the address from card
 *
 * returns: %TRUE if address has been successfully deleted, %FALSE otherwihe.
 */
gboolean 
rubrica_card_delete_address (RubricaCard* card, RubricaAddress* address)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(card), FALSE);
  g_return_val_if_fail(IS_RUBRICA_ADDRESS(address), FALSE);
  
  card->private->addr_iter = card->private->addresses;
  for (; card->private->addr_iter; 
       card->private->addr_iter = card->private->addr_iter->next)
    {
      if (card->private->addr_iter && 
	  (card->private->addr_iter->data == address))
	card->private->addresses = g_list_remove_link(card->private->addresses,
					     card->private->addr_iter);

      rubrica_address_free(address);
      g_list_free_1(card->private->addr_iter);
      card->private->addr_iter = NULL;

      return TRUE;
    }

  return FALSE;
}


/**
 * rubrica_card_replace_address
 * @card: a #RubricaCard
 * @old: a #RubricaAddress
 * @new: a #RubricaAddress
 * 
 * replace the old address with new one
 *
 * returns: %TRUE if address has been successfully replaced, %FALSE otherwihe.
 */
gboolean 
rubrica_card_replace_address (RubricaCard* card,
			      RubricaAddress* old, RubricaAddress* new)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(card), FALSE);
  g_return_val_if_fail(IS_RUBRICA_ADDRESS(old), FALSE);
  g_return_val_if_fail(IS_RUBRICA_ADDRESS(new), FALSE);

  // TODO

  return FALSE;
}


/**
 * rubrica_card_add_net_address 
 * @card: a #RubricaCard
 * @net: a #RubricaNetAddress
 *
 * add a net address to card 
 */
void 
rubrica_card_add_net_address (RubricaCard* card, RubricaNetAddress* net)
{
  g_return_if_fail(IS_RUBRICA_CARD(card));
  g_return_if_fail(IS_RUBRICA_NET_ADDRESS(net));

  card->private->nets = g_list_append(card->private->nets, net);

  /* init card iterator */
  if (!card->private->net_iter)
    card->private->net_iter = card->private->nets;
}


/**
 * rubrica_card_add_telephone
 * @card: a #RubricaCard
 * @tel: a #RubricaTelephone
 * 
 * add a telephone number to card
 */
void 
rubrica_card_add_telephone (RubricaCard* card, RubricaTelephone* tel)
{
  g_return_if_fail(IS_RUBRICA_CARD(card));
  g_return_if_fail(IS_RUBRICA_TELEPHONE(tel));

  card->private->phones = g_list_append(card->private->phones, tel);

  /* init telephone iterator */
  if (!card->private->phone_iter)
    card->private->phone_iter = card->private->phones;
}


/**
 * rubrica_card_get_group
 * @card: a #RubricaCard
 *
 * get the first of card's groups. User must cast returned value 
 * to a #RubricaGroup.
 * 
 * Returns: a gpointer.
 */
RubricaGroup* 
rubrica_card_get_group (RubricaCard* card)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(card), NULL);

  return rubrica_group_set_get_group(card->private->set);
}


/**
 * rubrica_card_get_next_group
 * @card: a #RubricaCard
 *
 * get the next in card's group. User must cast returned value 
 * to a #RubricaGroup.
 * 
 * Returns: a gpointer 
 */
RubricaGroup*
rubrica_card_get_next_group (RubricaCard* card)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(card),NULL);

  return rubrica_group_set_get_next_group(card->private->set);
}


/**
 * rubrica_card_get_prev_group
 * @card: a #RubricaCard
 *
 * get the previous in card's group. User must cast returned value 
 * to a #RubricaGroup.
 * 
 * Returns: a gpointer 
 */
RubricaGroup*
rubrica_card_get_prev_group (RubricaCard* card)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(card), NULL);

  return rubrica_group_set_get_prev_group(card->private->set);
}



/**
 * rubrica_card_reset_group
 * @card: a #RubricaCard
 *
 * set the private group iterartor to the first one
 */
void 
rubrica_card_reset_group (RubricaCard* card)
{
  g_return_if_fail(IS_RUBRICA_CARD(card));

  rubrica_group_set_reset(card->private->set);
}



/**
 * rubrica_card_find_group
 * @card: a #RubricaCard
 * @name: group's name
 *
 * find a card's group by name. User must cast returned value 
 * to a #RubricaGroup.
 *
 * Returns: a gpointer to the group if group has been found, NULL otherwise.
 */
gpointer 
rubrica_card_find_group (RubricaCard* card, gchar* name)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(card), NULL); 
  g_return_val_if_fail(name != NULL, NULL);

  return rubrica_group_set_find(card->private->set, name); 
} 


/** 
 * rubrica_card_belong_to_group
 * @card: a #RubricaCard
 * @group_name: group's name
 * 
 * verify if card belongs to the group 
 *
 * Returns: %TRUE if card belongs to the given group, %FALSE otherwise
 */ 
gboolean  
rubrica_card_belong_to_group (RubricaCard* card, gchar* group_name)
{ 
  g_return_val_if_fail(IS_RUBRICA_CARD(card), FALSE);  
  g_return_val_if_fail(group_name != NULL, FALSE); 
  
  return rubrica_group_set_owns_group(card->private->set, group_name); 
}



/**
 * rubrica_card_get_address
 * @card: a #RubricaCard
 *
 * get the first item of card's addresses list. User must cast to
 * a #RubricaAddress.
 *
 * Returns: a gpointer 
 */
gpointer
rubrica_card_get_address (RubricaCard* card)
{
  RubricaAddress* ret = NULL;

  g_return_val_if_fail(IS_RUBRICA_CARD(card), NULL);

  if (card->private->addr_iter)
    ret = card->private->addr_iter->data;
  if (IS_RUBRICA_ADDRESS(ret))
    return ret;
  
  return NULL;
}


/**
 * rubrica_card_get_next_address
 * @card: a #RubricaCard
 *
 * get the next item in card's addresses list. User must cast to
 * a #RubricaAddress.
 * 
 * Returns: a gpointer 
 */
gpointer 
rubrica_card_get_next_address (RubricaCard* card)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(card), NULL);
  
  card->private->addr_iter = g_list_next(card->private->addr_iter);
  if(card->private->addr_iter)
    return card->private->addr_iter->data;
  else  /* iterator is NULL, reset iterator */
    card->private->addr_iter = card->private->addresses;

  return NULL;
}


/**
 * rubrica_card_get_prev_address
 * @card: a #RubricaCard
 *
 * get the previous item in card's addresses list. User must cast to
 * a #RubricaAddress.
 * 
 * Returns: a gpointer 
 */
gpointer
rubrica_card_get_prev_address (RubricaCard* card)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(card), NULL);
  
  card->private->addr_iter = g_list_previous(card->private->addr_iter);
  if (card->private->addr_iter)
    return card->private->addr_iter->data;
  else  /* iterator is NULL, reset iterator */
    card->private->addr_iter = card->private->addresses;

  return NULL;
}


/**
 * rubrica_card_reset_address
 * @card: a #RubricaCard
 *
 * set the private address iterartor to the first one
 */
void 
rubrica_card_reset_address (RubricaCard* card)
{
  g_return_if_fail(IS_RUBRICA_CARD(card));

  card->private->addr_iter = card->private->addresses;
}


/**
 * rubrica_card_get_net_address
 * @card: a #RubricaCard
 *
 * get the first item of card's net addresses list. User must cast to
 * a #RubricaNetAddress.
 * 
 * Returns: a gpointer 
 */
gpointer
rubrica_card_get_net_address (RubricaCard* card)
{
  RubricaNetAddress* ret = NULL;

  g_return_val_if_fail(IS_RUBRICA_CARD(card), NULL);

  if (card->private->net_iter)
    ret = card->private->net_iter->data;
  if (IS_RUBRICA_NET_ADDRESS(ret))
    return ret;
  
  return NULL;
}


/**
 * rubrica_card_get_next_net_address
 * @card: a #RubricaCard
 *
 * get the next item in card's net addresses list. User must cast to
 * a #RubricaNetAddress.
 * 
 * Returns: a gpointer 
 */
gpointer
rubrica_card_get_next_net_address (RubricaCard* card)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(card), NULL);
  
  card->private->net_iter = g_list_next(card->private->net_iter);
  if (card->private->net_iter) 
    return card->private->net_iter->data;
  else  /* iterator is NULL, reset iterator */
    card->private->net_iter = card->private->nets;

  return NULL;  
}


/**
 * rubrica_card_get_prev_net_address
 * @card: a #RubricaCard
 *
 * get the previous item in card's net addresses list. User must cast to
 * a #RubricaNetAddress.
 * 
 * Returns: a gpointer 
 */
gpointer
rubrica_card_get_prev_net_address (RubricaCard* card)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(card), NULL);
  
  card->private->net_iter = g_list_previous(card->private->net_iter);
  if (card->private->net_iter)
    return card->private->net_iter->data;
  else
    card->private->net_iter = card->private->nets;

  return NULL;  
}


/**
 * rubrica_card_reset_net_address
 * @card: a #RubricaCard
 *
 * set the private net addresses iterartor to the first one.
 */
void 
rubrica_card_reset_net_address (RubricaCard* card)
{
  g_return_if_fail(IS_RUBRICA_CARD(card));
  
  card->private->net_iter = card->private->nets;
}



/**
 * rubrica_card_get_telephone
 * @card: a #RubricaCard
 *
 * get the first item of card's telephones list. User must cast to
 * a #RubricaTelephone.
 * 
 * Returns: a gpointer 
 */
gpointer
rubrica_card_get_telephone (RubricaCard* card)
{
  RubricaTelephone* ret = NULL;

  g_return_val_if_fail(IS_RUBRICA_CARD(card), NULL);

  if (card->private->phone_iter)
    ret = card->private->phone_iter->data;
  if (IS_RUBRICA_TELEPHONE(ret))
    return ret;
  
  return NULL;  
}



/**
 * rubrica_card_get_next_telephone
 * @card: a #RubricaCard
 *
 * get the next item in card's telephones list. User must cast to
 * a #RubricaTelephone.
 * 
 * Returns: a gpointer 
 */
gpointer
rubrica_card_get_next_telephone (RubricaCard* card)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(card), NULL);
  
  card->private->phone_iter = g_list_next(card->private->phone_iter);
  if (card->private->phone_iter)
    return card->private->phone_iter->data;
  else  /* iterator is NULL, reset iterator */
    card->private->phone_iter = card->private->phones;
  
  return NULL;  
}


/**
 * rubrica_card_get_prev_telephone
 * @card: a #RubricaCard
 *
 * get the previous item in card's telephones list. User must cast to
 * a #RubricaTelephone. 
 * 
 * Returns: a gpointer 
 */
gpointer
rubrica_card_get_prev_telephone (RubricaCard* card)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(card), NULL);
  
  card->private->phone_iter= g_list_previous(card->private->phone_iter);
  if (card->private->phone_iter)
    return card->private->phone_iter->data;
  else
    card->private->phone_iter = card->private->phones;

  return NULL;  
}



/**
 * rubrica_card_reset_telephone
 * @card: a #RubricaCard
 *
 * set the private telephones iterartor to the first one
 */
void 
rubrica_card_reset_telephone (RubricaCard* card)
{
  g_return_if_fail(IS_RUBRICA_CARD(card));
  
  card->private->phone_iter = card->private->phones;
}



/**
 * rubrica_card_get_group_owner
 * @card: a #RubricaCard
 * @group: a #RubricaGroup
 *
 * get the owner of the given group. Caller does not free the returned value
 * 
 * Returns: a gchar*, "rubrica" if given group if a default group, 
 * "user" if given group was build by user
 */
gchar*   
rubrica_card_get_group_owner (RubricaCard* card, RubricaGroup* group)
{
  gchar* owner;

  g_return_val_if_fail(IS_RUBRICA_CARD(card), NULL);
  g_return_val_if_fail(IS_RUBRICA_GROUP(card), NULL);
  
  g_object_get(G_OBJECT(group), "group-owner", &owner);
  
  return owner;
}



/**
 * rubrica_card_get_home_page
 * @card: a #RubricaCard
 *
 * get the first web url in net addresses list. 
 * Caller does not free the returned value
 * 
 * Returns: a gchar*
 */
gchar* 
rubrica_card_get_home_page (RubricaCard* card)
{
  gpointer net;

  g_return_val_if_fail(IS_RUBRICA_CARD(card), "");
  
  net = rubrica_card_get_net_address(card);
  for (; net; net = rubrica_card_get_next_net_address(card))
    {
      RubricaNetAddressType type = RUBRICA_NET_ADDRESS_UNKNOWN;
      gchar* url;

      g_object_get(RUBRICA_NET_ADDRESS(net), "url", &url, 
		   "url-type", &type, NULL);
      if (type == RUBRICA_NET_ADDRESS_WEB)
	{
	  rubrica_card_reset_net_address(card);
		  
	  return url;
	}
    }

  return "";
}


/**
 * rubrica_card_get_email
 * @card: a #RubricaCard
 *
 * get the first email in net addresses list. 
 * Caller does not free the returned value
 * 
 * Returns: a gchar*
 */
gchar* rubrica_card_get_email (RubricaCard* card)
{
  gpointer net;

  g_return_val_if_fail(IS_RUBRICA_CARD(card), "");
  
  net = rubrica_card_get_net_address(card);
  for (; net; net = rubrica_card_get_next_net_address(card))
    {
      RubricaNetAddressType type = RUBRICA_NET_ADDRESS_UNKNOWN;
      gchar* url;

      g_object_get(RUBRICA_NET_ADDRESS(net), "url", &url, 
		   "url-type", &type, NULL);
      if (type == RUBRICA_NET_ADDRESS_EMAIL)
	{
	  rubrica_card_reset_net_address(card);

	  return url;
	}
    }

  return "";
}


/**
 * rubrica_card_get_irc
 * @card: a #RubricaCard
 *
 * get the first irc in net addresses list. 
 * Caller does not free the returned value
 * 
 * Returns: a gchar*
 */
gchar* 
rubrica_card_get_irc (RubricaCard* card)
{
  gpointer net;

  g_return_val_if_fail(IS_RUBRICA_CARD(card), "");
  
  net = rubrica_card_get_net_address(card);
  for (; net; net = rubrica_card_get_next_net_address(card))
    {
      RubricaNetAddressType type = RUBRICA_NET_ADDRESS_UNKNOWN;
      gchar* url;

      g_object_get(net, "url", &url, "url-type", &type, NULL);
      if ((type >= RUBRICA_NET_ADDRESS_IRC) && 
	  (type <= RUBRICA_NET_ADDRESS_IRC_MSN))
	{
	  rubrica_card_reset_net_address(card);
	  
	  return url;
	}
    }

  return "";  
}


/* eof */
