/*
*  RAL -- Rubrica Addressbook Library
*  file: company.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <time.h>
#include <glib.h>
#include <glib-object.h>

#include "types.h"
#include "card.h"
#include "company.h" 
#include "date.h"
#include "group.h"
#include "address.h"
#include "net.h"
#include "telephone.h"
#include "utils.h"
#include "save.h"
#include "load.h"
#include "ref.h"


enum {
  COMPANY_NAME = 1,
  COMPANY_PI,
  COMPANY_NOTES
};


struct _RubricaCompanyCardPrivate {
  gchar* company_name;
  gchar* pi;                /* partita iva | codice fiscale */ 
  gchar* notes;                 

  GList* refs;
  GList* refs_iter;

  gboolean dispose_has_run;
};



static void rubrica_company_card_class_init (RubricaCompanyCardClass* klass);
static void rubrica_company_card_init       (RubricaCompanyCard* obj);

static void rubrica_company_card_dispose    (RubricaCompanyCard* obj);
static void rubrica_company_card_finalize   (RubricaCompanyCard* obj);

static void rubrica_company_card_set_property (GObject* obj, 
					       guint property_id,
					       const GValue* value, 
					       GParamSpec* spec);
static void rubrica_company_card_get_property (GObject* obj, 
					       guint property_id,
					       GValue* value, 
					       GParamSpec* spec);


static void rubrica_company_card_free         (RubricaCompanyCard* self); 

RubricaCompanyCard* rubrica_company_card_copy (RubricaCompanyCard* self);
RubricaInfos* rubrica_company_get_infos       (RubricaCompanyCard* self);

static RubricaCardClass* parent_class = NULL;



GType
rubrica_company_card_get_type()
{
  static GType company_card_type = 0;
  
  if (!company_card_type)
    {
      static const GTypeInfo company_card_info =
	{
	  sizeof(RubricaCompanyCardClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_company_card_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaCompanyCard),
	  0,
	  (GInstanceInitFunc) rubrica_company_card_init
	};

      company_card_type = g_type_register_static (RUBRICA_CARD_TYPE,
						  "RubricaCompanyCard",
						  &company_card_info, 0);
    }
  
  return company_card_type;
}


static void
rubrica_company_card_class_init(RubricaCompanyCardClass* klass)
{
  GObjectClass *class;
  GParamSpec* pspec;
  
  parent_class                  = RUBRICA_CARD_CLASS(klass);
  parent_class->free            = (gpointer) rubrica_company_card_free;
  parent_class->copy            = (gpointer) rubrica_company_card_copy;

  parent_class->write_infos     = (gpointer) rubrica_save_write_infos;
  parent_class->write_contact   = (gpointer) rubrica_save_write_company;
  parent_class->write_group     = (gpointer) rubrica_save_write_group;
  parent_class->write_addresses = (gpointer) rubrica_save_write_addresses;
  parent_class->write_net       = (gpointer) rubrica_save_write_net;
  parent_class->write_telephone = (gpointer) rubrica_save_write_telephone;

  parent_class->read_infos      = (gpointer) rubrica_load_read_infos;
  parent_class->read_contact    = (gpointer) rubrica_load_read_company;
  parent_class->read_group      = (gpointer) rubrica_load_read_groups;
  parent_class->read_addresses  = (gpointer) rubrica_load_read_addresses;
  parent_class->read_net        = (gpointer) rubrica_load_read_net;
  parent_class->read_telephone  = (gpointer) rubrica_load_read_telephone;

  parent_class->get_infos       = (gpointer) rubrica_company_get_infos;

  parent_class->print_card      = NULL;
   
  class           = G_OBJECT_CLASS (klass);
  class->dispose  = (GObjectFinalizeFunc) rubrica_company_card_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_company_card_finalize;

  class->set_property = rubrica_company_card_set_property;
  class->get_property = rubrica_company_card_get_property;
  
  /**
   * RubricaCompanyCard:company-name
   *
   * the name of the company
   */
  pspec = g_param_spec_string("company-name",
			      "company name",
			      "the name of the company",
			      NULL,
			      G_PARAM_READWRITE);
  g_object_class_install_property(class, COMPANY_NAME, pspec);

  // FIXME: trovare nome migliore
  /**
   * RubricaCompanyCard:pi
   *
   * Partita Iva (Codice Fiscale)
   */
  pspec = g_param_spec_string("pi",
			      "iva cf",
			      "partita iva o codice fiscale",
			      NULL, 
			      G_PARAM_READWRITE);
  g_object_class_install_property(class, COMPANY_PI, pspec);

  /**
   * RubricaCompanyCard:notes
   *
   * notes about the company
   */
  pspec = g_param_spec_string("notes",
			      "notes",
			      "company's notes",
			      NULL, 
			      G_PARAM_READWRITE);
  g_object_class_install_property(class, COMPANY_NOTES, pspec);
}


static void
rubrica_company_card_init(RubricaCompanyCard* self)
{
  self->private = g_new(RubricaCompanyCardPrivate, 1);

  self->private->company_name    = NULL;
  self->private->pi              = NULL;
  self->private->notes           = NULL;

  self->private->refs            = NULL;
  self->private->refs_iter       = NULL;
  self->private->dispose_has_run = FALSE;
}



static void 
rubrica_company_card_dispose (RubricaCompanyCard* self)
{
  g_return_if_fail(IS_RUBRICA_COMPANY_CARD(self));
    
  if (self->private->dispose_has_run)
    return;

  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_company_card_finalize (RubricaCompanyCard* self)
{
  g_return_if_fail(IS_RUBRICA_COMPANY_CARD(self));

  rubrica_utils_free_string(self->private->company_name);
  rubrica_utils_free_string(self->private->pi);  
  rubrica_utils_free_string(self->private->notes);

  for(; self->private->refs_iter; 
      self->private->refs_iter = self->private->refs_iter->next)
    if(self->private->refs_iter->data)
      rubrica_ref_free(RUBRICA_REF(self->private->refs_iter->data));

  g_list_free(self->private->refs);
  g_list_free(self->private->refs_iter);

  self->private->refs = self->private->refs_iter = NULL;
}



static void 
rubrica_company_card_set_property (GObject* obj, guint property_id,
				   const GValue* value, GParamSpec* spec)
{
  RubricaCompanyCard* self = (RubricaCompanyCard*) obj;
  
  switch (property_id)
    {
    case COMPANY_NAME:
      g_free(self->private->company_name);
      self->private->company_name = g_value_dup_string(value);
      break;
    
    case COMPANY_PI:
      g_free(self->private->pi);
      self->private->pi = g_value_dup_string(value);
      break;
      
    case COMPANY_NOTES:
      g_free(self->private->notes);
      self->private->notes = g_value_dup_string(value);
      break;
      
    default:
      break;
    }
}


static void 
rubrica_company_card_get_property (GObject* obj, guint property_id,
				   GValue* value, GParamSpec* spec)
{
  RubricaCompanyCard* self = (RubricaCompanyCard*) obj;
  
  switch (property_id)
    {
    case COMPANY_NAME:
      g_value_set_string(value, self->private->company_name);
      break;
    
    case COMPANY_PI:
      g_value_set_string(value, self->private->pi);
      break;
      
    case COMPANY_NOTES:
      g_value_set_string(value, self->private->notes);
      break;
      
    default:
      break;
    }  
}


/*  private functions
*/
static void
rubrica_company_card_free(RubricaCompanyCard* self)
{
  g_return_if_fail(IS_RUBRICA_COMPANY_CARD(self));

  g_object_unref(G_OBJECT(self));
}


RubricaCompanyCard* 
rubrica_company_card_copy (RubricaCompanyCard* self)
{
  RubricaCompanyCard* new;
  gpointer data;
  gchar *name, *pi, *notes;

  g_return_val_if_fail(IS_RUBRICA_COMPANY_CARD(self), NULL);
 
  new = rubrica_company_card_new();
  
  g_object_get(self, "company-name", &name, "pi", &pi, "notes", &notes, NULL);
  g_object_set(new, "company-name", name, "pi", pi, "notes", notes, NULL); 

  data = rubrica_company_card_get_ref(RUBRICA_COMPANY_CARD(self));
  for (; data; data = rubrica_company_card_get_next_ref(self))
    {
      RubricaRef* ref;

      ref = rubrica_ref_copy(RUBRICA_REF(data));
      if (ref)
	rubrica_company_card_add_ref(new, ref);
    }

  return new;  
}


RubricaInfos* 
rubrica_company_get_infos (RubricaCompanyCard* self)
{
  RubricaInfos* infos;

 infos = g_malloc0(sizeof(RubricaInfos));
 if (!infos)
   g_error("Unable to get enougth memory");
 
 g_object_get(self, "company-name", &infos->first, NULL);
 
 infos->last = infos->prof = infos->assignment = g_strdup(NULL);

 return infos;
}


/*  Public functions
*/

/**
 * rubrica_company_card_new
 * 
 * create a new #RubricaCompanyCard
 *
 * returns: a #RubricaCompanyCard*
 */
RubricaCompanyCard*
rubrica_company_card_new(void)
{
  RubricaCompanyCard* company;
  gulong id = 0L;
  gchar* cc_id = NULL;  /* Company Card id */

  company = g_object_new(rubrica_company_card_get_type(), NULL);

  id = RUBRICA_CARD_CLASS(parent_class)->generate_id(RUBRICA_CARD(company), 
						     "company");
  
  if (id == 0L)
    return NULL;

  cc_id = g_strdup_printf("cc%ld", id); 
  rubrica_card_set_id(RUBRICA_CARD(company), cc_id); 
  
  g_free(cc_id);
  cc_id = NULL;
  
  return company;
}




/**
 * rubrica_company_card_add_ref
 * @company: a #RubricaCompanyCard
 * @ref: a #RubricaRef
 *
 * add a #RubricaRef to company
 */
void 
rubrica_company_card_add_ref (RubricaCompanyCard* company, RubricaRef* ref)
{
  g_return_if_fail(IS_RUBRICA_COMPANY_CARD(company));
  g_return_if_fail(IS_RUBRICA_REF(ref));

  company->private->refs = g_list_append(company->private->refs, ref);
  if (!company->private->refs_iter)
    company->private->refs_iter = company->private->refs;
}


/**
 * rubrica_company_card_del_ref
 * @company: a #RubricaCompanyCard
 * @ref: a #RubricaRef
 *
 * remove a #RubricaRef from company 
 * *** Not yet implemented ***
 */
void 
rubrica_company_card_del_ref (RubricaCompanyCard* company, RubricaRef* ref)
{
  // todo
}


/**
 * rubrica_company_card_get_ref
 * @company: a #RubricaCompanyCard
 *
 * get the first #RubricaRef
 *
 * returns: a gpointer or NULL if company hasn't refs
 */
gpointer 
rubrica_company_card_get_ref (RubricaCompanyCard* company)
{
  g_return_val_if_fail(IS_RUBRICA_COMPANY_CARD(company), NULL);

  if (company->private->refs_iter)
    return company->private->refs_iter->data;
  
  return NULL;
}


/**
 * rubrica_company_card_get_next_ref
 * @company: a #RubricaCompanyCard
 *
 * get the next ref 
 *
 * returns: a gpointer or NULL
 */
gpointer 
rubrica_company_card_get_next_ref (RubricaCompanyCard* company)
{
  g_return_val_if_fail(IS_RUBRICA_COMPANY_CARD(company), NULL);

  company->private->refs_iter = g_list_next(company->private->refs_iter);
  if (company->private->refs_iter)
    return company->private->refs_iter->data;
  else
    company->private->refs_iter = company->private->refs;

  return NULL;
}


/**
 * rubrica_company_card_get_prev_ref 
 * @company: a #RubricaCompanyCard
 *
 * get the previous ref 
 *
 * returns: a gpointer or NULL 
 */
gpointer 
rubrica_company_card_get_prev_ref (RubricaCompanyCard* company)
{
  g_return_val_if_fail(IS_RUBRICA_COMPANY_CARD(company), NULL);

  company->private->refs_iter = g_list_previous(company->private->refs_iter);
  if (company->private->refs_iter)
    return company->private->refs_iter->data;
  else
    company->private->refs_iter = company->private->refs;

  return NULL;
}


/**
 * rubrica_company_card_reset_ref
 * @company: a #RubricaCompanyCard
 *
 * reset the private refs's iterator to ref head's list
 */
void     
rubrica_company_card_reset_ref (RubricaCompanyCard* company)
{
  g_return_if_fail(IS_RUBRICA_COMPANY_CARD(company));

  company->private->refs_iter = company->private->refs;
}
