/*
*  RAL -- Rubrica Addressbook Library
*  file: date.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <time.h>
#include <stdlib.h>
#include <glib.h>
#include <glib-object.h>

#include "date.h"
#include "utils.h"

#define BUFLEN 64

/*   property enumeration
*/
enum {
  DATE_KNOW = 1,
  DATE_DAY,
  DATE_MONTH,
  DATE_YEAR,
  DATE_HUMAN
};

struct _RubricaDatePrivate {
  gboolean know;      /* is date khnown? */

  gchar* day;                
  gchar* month;
  gchar* year; 

  gchar* human_date;

  gboolean dispose_has_run;
};



static void rubrica_date_class_init (RubricaDateClass* klass);
static void rubrica_date_init       (RubricaDate* obj);

static void rubrica_date_dispose    (RubricaDate* obj);
static void rubrica_date_finalize   (RubricaDate* obj);


static void rubrica_date_set_property (GObject* obj, guint property_id,
				       const GValue* value, GParamSpec* spec);

static void rubrica_date_get_property (GObject* obj, guint property_id,
				       GValue* value, GParamSpec* spec);


GType
rubrica_date_get_type()
{
  static GType rubrica_date_type = 0;
  
  if (!rubrica_date_type)
    {
      static const GTypeInfo rubrica_date_info =
	{
	  sizeof(RubricaDateClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_date_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaDate),
	  0,
	  (GInstanceInitFunc) rubrica_date_init
	};

      rubrica_date_type = g_type_register_static (G_TYPE_OBJECT, "RubricaDate",
						  &rubrica_date_info, 0);
    }
  
  return rubrica_date_type;
}


static void
rubrica_date_class_init(RubricaDateClass* klass)
{
  GObjectClass *class;
  GParamSpec* pspec;
  
  class  = G_OBJECT_CLASS (klass);

  class->dispose  = (GObjectFinalizeFunc) rubrica_date_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_date_finalize;
  
  class->set_property = rubrica_date_set_property;
  class->get_property = rubrica_date_get_property;

  /**
   * RubricaDate:know:
   *
   * 
   */
  pspec = g_param_spec_boolean("know", "know", "is date known", FALSE,
			       G_PARAM_READWRITE);
  g_object_class_install_property (class, DATE_KNOW, pspec);

  /**
   * RubricaDate:day:
   *
   * day
   */
  pspec = g_param_spec_string("day", "day", "day", NULL, G_PARAM_READABLE);
  g_object_class_install_property (class, DATE_DAY, pspec);

  /**
   * RubricaDate:month:
   *
   * month
   */
  pspec = g_param_spec_string("month", "month", "month", 
			      NULL, G_PARAM_READABLE);
  g_object_class_install_property (class, DATE_MONTH, pspec);

  /**
   * RubricaDate:year:
   *
   * year
   */
  pspec = g_param_spec_string("year", "year", "year", NULL, G_PARAM_READABLE);
  g_object_class_install_property (class, DATE_YEAR, pspec);

  /**
   * RubricaDate:human-date:
   *
   * date in a human readable format
   */
  pspec = g_param_spec_string("human-date",
			      "human readable",
			      "data displayed in a human readable format",
			      NULL, G_PARAM_READABLE);
  g_object_class_install_property (class, DATE_HUMAN, pspec);
}


static void
rubrica_date_init(RubricaDate* self)
{
  self->private = g_new(RubricaDatePrivate, 1);

  self->private->know   = FALSE;

  self->private->day    = g_strdup("BadDay");   
  self->private->month  = g_strdup("BadMonth");
  self->private->year   = g_strdup("BadYear");

  self->private->human_date  = NULL;

  self->private->dispose_has_run = FALSE;
}


static void 
rubrica_date_dispose (RubricaDate* self)
{
  g_return_if_fail(IS_RUBRICA_DATE(self));
  
#ifdef DEBUG
  g_print("\n\t\trubrica_date_dispose");
#endif
  
  if (self->private->dispose_has_run)
    return;

  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_date_finalize (RubricaDate* self)
{
  g_return_if_fail(IS_RUBRICA_DATE(self));

#ifdef DEBUG
  g_print("\n\t\trubrica_date_finalize");
#endif

  rubrica_utils_free_string(self->private->day);
  rubrica_utils_free_string(self->private->month);
  rubrica_utils_free_string(self->private->year);
  rubrica_utils_free_string(self->private->human_date);

  g_free(self->private);
  self->private = NULL;
}


static void 
rubrica_date_set_property (GObject* obj, guint property_id,
			   const GValue* value,  GParamSpec* spec)
{
  /* do nothing */ 
  return;
}


static void 
rubrica_date_get_property (GObject* obj, guint property_id,
			   GValue* value, GParamSpec* spec)
{
  RubricaDate *self = (RubricaDate *) obj;

  switch(property_id)
    {
    case DATE_KNOW:
      g_value_set_boolean(value, self->private->know);
      break;
      
    case DATE_DAY:
      g_value_set_string(value, self->private->day);
      break;
      
    case DATE_MONTH:
      g_value_set_string(value, self->private->month);
      break;
      
    case DATE_YEAR:
      g_value_set_string(value, self->private->year);
      break;
      
    case DATE_HUMAN:
      g_value_set_string(value, self->private->human_date);
      break;
      
    default:
      break;
    }
}



/**
 * rubrica_date_new
 *
 * create a new #RubricaDate object
 *
 * Returns: a #RubricaDate
 **/
RubricaDate*
rubrica_date_new(void)
{
  RubricaDate* date;

  date = g_object_new(rubrica_date_get_type(), NULL);

  return date;
}


/**
 * rubrica_date_free
 * @date: a #RubricaDate
 *
 * free the RubricaDate object
 */
void
rubrica_date_free(RubricaDate* date)
{
  g_return_if_fail(IS_RUBRICA_DATE(date));

#ifdef DEBUG
  g_print("\n\t\t**** Free Date");
#endif

  g_object_unref(date);   
}


/**
 * rubrica_date_know_date
 * @date: a #RubricaDate
 * @know: gboolean
 *
 * Set the know date flag. Set to %TRUE if the date is known, %FALSE otherwise
 */
void 
rubrica_date_know_date (RubricaDate* date, gboolean know)
{
  g_return_if_fail(IS_RUBRICA_DATE(date));

  date->private->know = know;
}


/**
 * rubrica_date_set_date
 * @date: a #RubricaDate
 * @day:
 * @month:
 * @year:
 *
 * set the data values.
 */
void 
rubrica_date_set_date (RubricaDate* date, gint day, gint month, gint year)
{
  struct tm *tm;
  time_t t;
  gchar buffer[BUFLEN];

  g_return_if_fail(IS_RUBRICA_DATE(date));
  
  date->private->know = FALSE;
  rubrica_utils_free_string(date->private->day);
  rubrica_utils_free_string(date->private->month);
  rubrica_utils_free_string(date->private->year);

  date->private->day   = g_strdup("BadDay");   
  date->private->month = g_strdup("BadMonth");
  date->private->year  = g_strdup("BadYear");
  
  if ((day < 0) || (day > 31))
    return;
  
  /* 0 < tm_mon < 11 ==> 0 is january, 11 is december */
  if ((month < 0) || (month > 12)) 
    return;
    
  date->private->know = TRUE;

  t = time(NULL);
  tm = localtime(&t);
  tm->tm_mday = day;
  tm->tm_mon  = month - 1;
  tm->tm_year = year;

  rubrica_utils_free_string(date->private->day);
  rubrica_utils_free_string(date->private->month);
  rubrica_utils_free_string(date->private->year);
  
  date->private->day   = g_strdup_printf("%d", tm->tm_mday);
  date->private->month = g_strdup_printf("%d", tm->tm_mon + 1);
  date->private->year  = g_strdup_printf("%d", tm->tm_year);  
  
  strftime(buffer, BUFLEN, "%Ex", tm);
  
  rubrica_utils_free_string(date->private->human_date);
  date->private->human_date = g_strdup(buffer);
}
 


/**
 * rubrica_date_get_human_date
 * @date: a #RubricaDate
 *
 * get the date as a locale string. Caller must free the returned value
 *
 * returns: a gchar* or "unknown", if date is not set
 */
gchar*       
rubrica_date_get_human_date (RubricaDate* date)
{
  g_return_val_if_fail(IS_RUBRICA_DATE(date), g_strdup(_("unknown")));

  return g_strdup(date->private->human_date);
}


/**
 * rubrica_date_is_date_known
 * @date: a #RubricaDate
 *
 * get the state of know date flag
 *
 * returns: a boolean
 */ 
gboolean 
rubrica_date_is_date_known (RubricaDate* date)
{
  g_return_val_if_fail(IS_RUBRICA_DATE(date), FALSE);

  return date->private->know;
}



/**
 * rubrica_date_copy
 * @date: a #RubricaDate
 * 
 * copy the given #RubricaDate
 * 
 * returns: a new allocated RubricaDate*
 */
RubricaDate* 
rubrica_date_copy (RubricaDate* date)
{
  RubricaDate* new;
  gchar *day, *month, *year;

  g_return_val_if_fail(IS_RUBRICA_DATE(date), NULL);

  new = rubrica_date_new();
  
  g_object_get(date, "day", &day, "month", &month, "year", &year, NULL);

  rubrica_date_know_date(new, date->private->know);
  rubrica_date_set_date(new, atoi(day), atoi(month), atoi(year));

  return new;
}
