/*
*  RAL -- Rubrica Addressbook Library
*  file: groups.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <time.h>
#include <glib.h>
#include <glib-object.h>

#include "group_set.h"
#include "group.h"

enum {
  GROUP_ADDED,
  GROUP_REMOVED,

  LAST_SIGNAL 
};

static guint rubrica_group_set_signals[LAST_SIGNAL] = {0};


struct _RubricaGroupSetPrivate {
  GList* lst;
  GList* iter;
  GList* find;

  gboolean dispose_has_run;
};


static void rubrica_group_set_class_init (RubricaGroupSetClass* klass);
static void rubrica_group_set_init       (RubricaGroupSet* obj);

static void rubrica_group_set_dispose    (RubricaGroupSet* obj);
static void rubrica_group_set_finalize   (RubricaGroupSet* obj);



GType
rubrica_group_set_get_type()
{
  static GType group_set_type = 0;
  
  if (!group_set_type)
    {
      static const GTypeInfo group_set_info =
	{
	  sizeof(RubricaGroupSetClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_group_set_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaGroupSet),
	  0,
	  (GInstanceInitFunc) rubrica_group_set_init
	};

      group_set_type = g_type_register_static (G_TYPE_OBJECT, 
					       "RubricaGroupSet",
					       &group_set_info, 0);
    }
  
  return group_set_type;
}


static void
rubrica_group_set_class_init(RubricaGroupSetClass* klass)
{
  GObjectClass *class; 

  class           = G_OBJECT_CLASS (klass);
  class->dispose  = (GObjectFinalizeFunc) rubrica_group_set_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_group_set_finalize;


  /* class signals */
  rubrica_group_set_signals[GROUP_ADDED] =
    g_signal_new("group_added", 
		 RUBRICA_GROUP_SET_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RubricaGroupSetClass, group_added),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__INT,
		 G_TYPE_NONE,            /* return type */
		 1,                      /* params      */
		 G_TYPE_POINTER);        /* params type */

  rubrica_group_set_signals[GROUP_REMOVED] =
    g_signal_new("group_removed", 
		 RUBRICA_GROUP_SET_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RubricaGroupSetClass, group_removed),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__INT,
		 G_TYPE_NONE,            /* return type */
		 1,                      /* params      */
		 G_TYPE_POINTER);        /* params type */
}


static void
rubrica_group_set_init(RubricaGroupSet* self)
{
  self->private = g_new(RubricaGroupSetPrivate, 1);

  self->private->lst  = NULL;
  self->private->iter = NULL;
  self->private->dispose_has_run = FALSE;
}



static void 
rubrica_group_set_dispose (RubricaGroupSet* self)
{
  GList* list;

  g_return_if_fail(IS_RUBRICA_GROUP_SET(self));
  
  if (self->private->dispose_has_run)
    return;

  for (list = self->private->lst; list; list = g_list_next(list))
    rubrica_group_free(RUBRICA_GROUP(list->data));   
  
  g_list_free(self->private->lst);

  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_group_set_finalize (RubricaGroupSet* self)
{
  g_return_if_fail(IS_RUBRICA_GROUP_SET(self));
  
  g_free(self->private);
  self->private = NULL;
}



/*    Public
*/

/**
 * rubrica_group_set_
 * @set: a #RubricaGroupSet
 *
 * create a new #RubricaGroupSet
 *
 * returns: a new allocated #RubricaGroupSet*
 */
RubricaGroupSet*
rubrica_group_set_new(void)
{
  RubricaGroupSet* set;

  set = g_object_new(rubrica_group_set_get_type(), NULL);

  return set;
}


/**
 * rubrica_group_set_free
 * @set: a #RubricaGroupSet
 *
 * free memory owned by a #RubricaGroupSet
 */
void
rubrica_group_set_free(RubricaGroupSet* set)
{
  g_return_if_fail(IS_RUBRICA_GROUP_SET(set));

  g_object_unref(set);   
}


/**
 * rubrica_group_set_find
 * @set: a #RubricaGroupSet
 * @group_name: a #RubricaGroup's name
 *
 * find a group by name
 * 
 * returns: a gpoiter, to the group with the given name or NULL, if 
 * set haven't the group. 
 */
gpointer  
rubrica_group_set_find (RubricaGroupSet* set, gchar* group_name) 
{ 
  g_return_val_if_fail(IS_RUBRICA_GROUP_SET(set), NULL);  
  g_return_val_if_fail(group_name != NULL, NULL); 

  set->private->find = set->private->lst;
  for (; set->private->find; set->private->find = set->private->find->next) 
    {
      RubricaGroup* grp = set->private->find->data;

      if (rubrica_group_has_name(RUBRICA_GROUP(grp), group_name)) 
	return (gpointer) grp;
    } 
  
  return NULL;
}


/**
 * rubrica_group_set_owns_group
 * @set: a #RubricaGroupSet
 * @group_name: a #RubricaGroup's name
 *
 * test if set owns the given group
 *
 * returns: a gboolean, %TRUE if set owns group, %FALSE otherwise
 */
gboolean 
rubrica_group_set_owns_group (RubricaGroupSet* set, gchar* group_name)
{
  g_return_val_if_fail(IS_RUBRICA_GROUP_SET(set), FALSE); 
  g_return_val_if_fail(group_name != NULL, FALSE); 

  if (g_ascii_strcasecmp(group_name, "all groups") == 0) 
    return TRUE; 

  if (rubrica_group_set_find(set, group_name)) 
    return TRUE; 
  
  return FALSE;   
}


/**
 * rubrica_group_set_add_group
 * @set: a #RubricaGroupSet
 * @group: a #RubricaGroup
 *
 * add the group to te set
 *
 * returns: a gboolean. %TRUE if group if added successfully, %FALSE otherwise
 */
gboolean               
rubrica_group_set_add_group (RubricaGroupSet* set, RubricaGroup* group) 
{
  gchar* name; 

  g_return_val_if_fail(IS_RUBRICA_GROUP_SET(set), FALSE); 
  g_return_val_if_fail(IS_RUBRICA_GROUP(group), FALSE); 

  /* test if group is already known 
   */
  g_object_get(group, "group-name", &name, NULL);
  if (rubrica_group_set_owns_group(set, name))
    return TRUE;
  
  /* else add to the set
   */
  set->private->lst = g_list_append(set->private->lst, group);

  /* init group iterator */
  if (!set->private->iter)
    set->private->iter = set->private->lst;  

  g_signal_emit_by_name(set, "group_added", group, G_TYPE_POINTER);

  return TRUE;
}


/**
 * rubrica_group_set_delete_group_by_name
 * @set: a #RubricaGroupSet
 * @name: group's name
 *
 * delete a group from set
 *
 * returns: a gboolean. %TRUE if group is deleted, %FALSE otherwise
 */
gboolean     
rubrica_group_set_delete_group_by_name (RubricaGroupSet* set, gchar* name)
{
  g_return_val_if_fail(IS_RUBRICA_GROUP_SET(set), FALSE); 
  g_return_val_if_fail(name != NULL, FALSE);
 
  set->private->iter = set->private->lst;
  for (; set->private->iter; set->private->iter = set->private->iter->next) 
    { 
      RubricaGroup* group = set->private->iter->data;
      
      if (rubrica_group_has_name(RUBRICA_GROUP(group), name)) 
	{ 
	  set->private->lst = g_list_remove_link(set->private->lst,
						  set->private->iter);
	  
	  rubrica_group_free(RUBRICA_GROUP(set->private->iter->data));
	  
	  g_list_free_1(set->private->iter);
	  set->private->iter = NULL;	  

	  return TRUE;
	}
    }
  
  return FALSE;
}


/**
 * rubrica_group_set_delete_group
 * @set: a #RubricaGroupSet
 * @group: a #RubricaGroup
 *
 * delete a group from set
 *
 * returns: a gboolean. %TRUE if group is deleted, %FALSE otherwise
 */
gboolean     
rubrica_group_set_delete_group (RubricaGroupSet* set, RubricaGroup* group)
{
  gchar* name;

  g_return_val_if_fail(IS_RUBRICA_GROUP_SET(set), FALSE); 
  g_return_val_if_fail(IS_RUBRICA_GROUP(group), FALSE);

  g_object_get(group, "group-name", &name, NULL);
 
  return rubrica_group_set_delete_group_by_name(set, name);
}


/**
 * rubrica_group_set_rename_group
 * @set: a #RubricaGroupSet
 * @oldname:
 * @newname:
 *
 * rename a group in the set
 *
 * returns: a gboolean. %TRUE if group is renamed, %FALSE otherwise
 */
gboolean    
rubrica_group_set_rename_group (RubricaGroupSet* set, gchar* oldname,
				gchar* newname)
{
  RubricaGroup* group;

  g_return_val_if_fail(IS_RUBRICA_GROUP_SET(set), FALSE); 
  g_return_val_if_fail(oldname != NULL, FALSE);
  g_return_val_if_fail(newname != NULL, FALSE);

  group = rubrica_group_set_find(set, oldname);
  if (rubrica_group_rename(group, newname))
    return TRUE;
  
  return FALSE;
}



/**
 * rubrica_group_set_disable_all
 * @set: a #RubricaGroupSet
 *
 * disable all groups in set 
 */
void     
rubrica_group_set_disable_all (RubricaGroupSet* set)
{
  RubricaGroup* group;

  g_return_if_fail(IS_RUBRICA_GROUP_SET(set)); 
  
  rubrica_group_set_reset(set);
  group = rubrica_group_set_get_group(set);
  for(; group; group = rubrica_group_set_get_next_group(set))
    g_object_set(group, "enabled", FALSE, NULL);
}


/**
 * rubrica_group_set_enable_group
 * @set: a #RubricaGroupSet
 * @group: a #RubricaGroup
 * 
 * enable the given group
 */
void     
rubrica_group_set_enable_group (RubricaGroupSet* set, RubricaGroup* grp)
{
  gchar* name;
  RubricaGroup* group = NULL;

  g_return_if_fail(IS_RUBRICA_GROUP_SET(set)); 
  g_return_if_fail(IS_RUBRICA_GROUP(grp));

  g_object_get(grp, "group-name", &name, NULL);
  group = rubrica_group_set_find(set, name);

  if (group)
    g_object_set(group, "enabled", TRUE, NULL);
}


/**
 * rubrica_group_set_disable_group
 * @set: a #RubricaGroupSet
 * @group: a #RubricaGroup
 * 
 * disable the given group
 */
void     
rubrica_group_set_disable_group (RubricaGroupSet* set, RubricaGroup* grp)
{
  gchar* name;
  RubricaGroup* group = NULL;

  g_return_if_fail(IS_RUBRICA_GROUP_SET(set)); 
  g_return_if_fail(IS_RUBRICA_GROUP(grp));
  
  g_object_get(grp, "group-name", &name, NULL);
  group = rubrica_group_set_find(set, name);
  if (group)
    g_object_set(group, "enabled", FALSE, NULL); 
}




/**
 * rubrica_group_set_merge_sets
 * @set: a #RubricaGroupSet
 * @second: a #RubricaGroupSet
 *
 * merges the #RubricaGroupSets
 *
 *returns: a RubricaGroupSet*
 */
RubricaGroupSet* 
rubrica_group_set_merge_sets (RubricaGroupSet* set, RubricaGroupSet* second)
{
  g_return_val_if_fail(IS_RUBRICA_GROUP_SET(set), NULL); 
  g_return_val_if_fail(IS_RUBRICA_GROUP_SET(second), set);
 
  set->private->lst = g_list_concat(set->private->lst, second->private->lst);
  
  return set;
}


/**
 * rubrica_group_set_reset
 * @set: a #RubricaGroupSet
 *
 * reset the private #RubricaGroupSet iterator to the head of group's list.
 */
void
rubrica_group_set_reset(RubricaGroupSet* set)
{
  g_return_if_fail(IS_RUBRICA_GROUP_SET(set));
  
  set->private->iter = set->private->lst;
}


/**
 * rubrica_group_set_get_group
 * @set: a #RubricaGroupSet
 *
 * get the first group in set
 *
 * returns: a gpointer or NULL if set is void
 */
RubricaGroup* 
rubrica_group_set_get_group (RubricaGroupSet* set)
{
  g_return_val_if_fail(IS_RUBRICA_GROUP_SET(set), NULL); 
  
  if (set->private->iter)
    return (RubricaGroup*) set->private->iter->data;
  
  return NULL;
}


/**
 * rubrica_group_set_get_next_group
 * @set: a #RubricaGroupSet
 *
 * get the next group in set. 
 *
 * returns: a gpointer or NULL if the groups's end list is reached.
 */
RubricaGroup* 
rubrica_group_set_get_next_group (RubricaGroupSet* set)
{
  g_return_val_if_fail(IS_RUBRICA_GROUP_SET(set), NULL); 

  set->private->iter = g_list_next(set->private->iter);
  if (set->private->iter)
    return (RubricaGroup*) set->private->iter->data;
  else
    set->private->iter = set->private->lst;

  return NULL;
}


/**
 * rubrica_group_set_get_prev_group
 * @set: a #RubricaGroupSet
 *
 * get the previous group in set.
 *
 * returns: a gpointer or NULL if the groups's head list is reached.
 */
RubricaGroup* 
rubrica_group_set_get_prev_group (RubricaGroupSet* set)
{
  g_return_val_if_fail(IS_RUBRICA_GROUP_SET(set), NULL); 

  g_return_val_if_fail(IS_RUBRICA_GROUP_SET(set), NULL); 

  set->private->iter = g_list_previous(set->private->iter);
  if (set->private->iter)
    return (RubricaGroup*) set->private->iter->data;
  else
    set->private->iter = set->private->lst;

  return NULL;
}







