/*
*  RAL -- Rubrica Addressbook Library
*  file: net.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#include <glib.h>
#include <glib-object.h>

#include "net.h"
#include "utils.h"

enum {
  NET_URL = 1,
  NET_TYPE
};

typedef struct _NetConversion {
  gint   token;
  gchar* label;
} NetConversion;


struct _RubricaNetAddressPrivate {
  gchar* url;
  gint   type;

  gboolean dispose_has_run;
};


static NetConversion table[] = {
  {RUBRICA_NET_ADDRESS_WEB,          "web"         },
  {RUBRICA_NET_ADDRESS_EMAIL,        "email"       },
  {RUBRICA_NET_ADDRESS_GNOMEMEETING, "gnomemeeting"},
  {RUBRICA_NET_ADDRESS_IRC,          "irc"         },
  {RUBRICA_NET_ADDRESS_IRC_AIM,      "aim"         },
  {RUBRICA_NET_ADDRESS_IRC_ICQ,      "icq"         },
  {RUBRICA_NET_ADDRESS_IRC_JABBER,   "jabber"      },
  {RUBRICA_NET_ADDRESS_IRC_YAHOO,    "yahoo"       },
  {RUBRICA_NET_ADDRESS_IRC_MSN,      "msn"         },
  {RUBRICA_NET_ADDRESS_UNKNOWN,      "unknown"     },
};



static void rubrica_net_address_class_init (RubricaNetAddressClass* klass);
static void rubrica_net_address_init       (RubricaNetAddress* obj);

static void rubrica_net_address_dispose    (RubricaNetAddress* obj);
static void rubrica_net_address_finalize   (RubricaNetAddress* obj);


static void rubrica_net_address_set_property (GObject* obj, guint property_id,
					      const GValue* value, 
					      GParamSpec* spec);
static void rubrica_net_address_get_property (GObject* obj, guint property_id,
					      GValue* value, GParamSpec* spec);


GType
rubrica_net_address_get_type()
{
  static GType rubrica_net_address_type = 0;
  
  if (!rubrica_net_address_type)
    {
      static const GTypeInfo rubrica_net_address_info =
	{
	  sizeof(RubricaNetAddressClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_net_address_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaNetAddress),
	  0,
	  (GInstanceInitFunc) rubrica_net_address_init
	};

      rubrica_net_address_type = 
	g_type_register_static (G_TYPE_OBJECT, "RubricaNetAddress",
				&rubrica_net_address_info, 0);
    }
  
  return rubrica_net_address_type;
}


static void
rubrica_net_address_class_init(RubricaNetAddressClass* klass)
{
  GObjectClass *class;
  GParamSpec* pspec;
  
  class  = G_OBJECT_CLASS (klass);

  class->dispose  = (GObjectFinalizeFunc) rubrica_net_address_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_net_address_finalize;
  
  class->set_property = rubrica_net_address_set_property;
  class->get_property = rubrica_net_address_get_property;

  /**
   * RubricaNetAddress:url:
   *
   * the net address uri
   */
  pspec = g_param_spec_string("url",
			      "url",
			      "the net address uri",
			      NULL,
			      G_PARAM_READWRITE);
  g_object_class_install_property(class, NET_URL, pspec);

  /**
   * RubricaNetAddress:url-type:
   *
   * the uri's type, see #RubricaNetAddressType for valid values
   */
  pspec = g_param_spec_int("url-type",
			   "url's type",
			   "url's type",
			   RUBRICA_NET_ADDRESS_WEB,
			   RUBRICA_NET_ADDRESS_UNKNOWN,
			   RUBRICA_NET_ADDRESS_UNKNOWN,
			   G_PARAM_READWRITE);
  g_object_class_install_property(class, NET_TYPE, pspec);
}



static void
rubrica_net_address_init(RubricaNetAddress* self)
{
  self->private = g_new(RubricaNetAddressPrivate, 1);

  self->private->url  = NULL;

  self->private->dispose_has_run = FALSE;
}


static void 
rubrica_net_address_dispose (RubricaNetAddress* self)
{
  g_return_if_fail(IS_RUBRICA_NET_ADDRESS(self));
  
  if (self->private->dispose_has_run)
    return;

  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_net_address_finalize (RubricaNetAddress* self)
{
  g_return_if_fail(IS_RUBRICA_NET_ADDRESS(self));

  rubrica_utils_free_string(self->private->url);

  g_free(self->private);
  self->private = NULL;
}


static void 
rubrica_net_address_set_property (GObject* obj, guint property_id,
				  const GValue* value, GParamSpec* spec)
{
  RubricaNetAddress* self = (RubricaNetAddress*) obj;
  
  switch (property_id)
    {
    case NET_URL:
      g_free(self->private->url);
      self->private->url = g_value_dup_string(value);
      break;

    case NET_TYPE:
      self->private->type = g_value_get_int(value);
      break;

    default:
      break;
    }
}


static void 
rubrica_net_address_get_property (GObject* obj, guint property_id,
				  GValue* value, GParamSpec* spec)
{
  RubricaNetAddress* self = (RubricaNetAddress*) obj;
  
  switch (property_id)
    {
    case NET_URL:
      g_value_set_string(value, self->private->url);
      break;

    case NET_TYPE:
      g_value_set_int(value, self->private->type);
      break;

    default:
      break;
    }
}


/**
 * rubrica_net_address_new
 *
 * create a new #RubricaNetAddress
 *
 * returns: a #RubricaNetAddress*
 */
RubricaNetAddress*
rubrica_net_address_new(void)
{
  RubricaNetAddress* net; 

  net = g_object_new(rubrica_net_address_get_type(), NULL); 

  return net; 
}


/**
 * rubrica_net_address_free
 * @net: a #RubricaNetAddress
 *
 * free the memory owned by net
 */
void
rubrica_net_address_free(RubricaNetAddress* net)
{
  g_return_if_fail(IS_RUBRICA_NET_ADDRESS(net));

  g_object_unref(net);   
}



/**
 * rubrica_net_address_copy
 * @net: a #RubricaNetAddress
 *
 * copy the given net address.
 *
 * returns: a new allocated #RubricaNetAddress*
 */
RubricaNetAddress* 
rubrica_net_address_copy (RubricaNetAddress *net)
{
  RubricaNetAddress* new;
  RubricaNetAddressType type;
  gchar *url;

  g_return_val_if_fail(IS_RUBRICA_NET_ADDRESS(net), NULL);

  new = rubrica_net_address_new();
  
  g_object_get(G_OBJECT(net), "url", &url, "url-type", &type, NULL);  
  g_object_set(G_OBJECT(net), "url", url, "url-type", type, NULL);

  return new;
}


/**
 * rubrica_net_address_decode_type
 * @type: a #RubricaNetAddressType
 * 
 * decode the #RubricaNetAddressType given type
 *
 * returns: a gchar*
 */
gchar* 
rubrica_net_address_decode_type (RubricaNetAddressType type)
{
  if ((type < RUBRICA_NET_ADDRESS_WEB) || (type > RUBRICA_NET_ADDRESS_UNKNOWN))
    return "unknown";

  return table[type].label;
}


/**
 * rubrica_net_address_get_decoded_type
 * @type: gchar*
 * 
 * decode the string returned by rubrica_net_address_get_url_type()
 *
 * returns: a #RubricaNetAddressType
 */
RubricaNetAddressType 
rubrica_net_address_encode_type (gchar* type)
{
  RubricaNetAddressType ret = RUBRICA_NET_ADDRESS_UNKNOWN;

  if (!type)
    return ret;

  if (g_ascii_strcasecmp(type, "web") == 0)
    ret = RUBRICA_NET_ADDRESS_WEB;
  else if (g_ascii_strcasecmp(type, "email") == 0)
    ret = RUBRICA_NET_ADDRESS_EMAIL;
  else if (g_ascii_strcasecmp(type, "gnomemeeting") == 0)
    ret = RUBRICA_NET_ADDRESS_GNOMEMEETING;
  else if (g_ascii_strcasecmp(type, "irc") == 0)
    ret = RUBRICA_NET_ADDRESS_IRC;
  else if (g_ascii_strcasecmp(type, "aim") == 0)
    ret = RUBRICA_NET_ADDRESS_IRC_AIM;
  else if (g_ascii_strcasecmp(type, "jabber") == 0)
    ret = RUBRICA_NET_ADDRESS_IRC_JABBER;
  else if (g_ascii_strcasecmp(type, "icq") == 0)
    ret = RUBRICA_NET_ADDRESS_IRC_ICQ;
  else if (g_ascii_strcasecmp(type, "yahoo") == 0)
    ret = RUBRICA_NET_ADDRESS_IRC_YAHOO;
  else if (g_ascii_strcasecmp(type, "msn") == 0)
    ret = RUBRICA_NET_ADDRESS_IRC_MSN;
  
  return ret;
}

