/*
*  RAL -- Rubrica Addressbook Library
*  file: notes.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdlib.h>
#include <glib.h>
#include <glib-object.h>

#include "notes.h"
#include "date.h"
#include "utils.h"

enum {
  NOTES_HAS_PARTNER = 1,
  NOTES_PARTNER,
  NOTES_CHILDREN,
  NOTES_OTHER_NOTES,                // hobbies and other notes
  NOTES_PUBKEY
};


struct _RubricaNotesPrivate {
  gboolean has_partner;      /* have a partner?    */

  gchar* partner;            /* partner's name     */
  gchar* children;           /* children notes     */
  gchar* notes;              /* other notes        */
  gchar* pubkey;             /* public key         */

  RubricaDate* birthday;     /* partner's birthday */
  RubricaDate* anniversary;  /* their anniversary  */

  gboolean dispose_has_run;
};



static void rubrica_notes_class_init (RubricaNotesClass* klass);
static void rubrica_notes_init       (RubricaNotes* obj);

static void rubrica_notes_dispose    (RubricaNotes* obj);
static void rubrica_notes_finalize   (RubricaNotes* obj);


static void rubrica_notes_set_property (GObject* obj, guint property_id,
					const GValue* value, 
					GParamSpec* spec);
static void rubrica_notes_get_property (GObject* obj, guint property_id,
					GValue* value, GParamSpec* spec);

GType
rubrica_notes_get_type()
{
  static GType notes_type = 0;
  
  if (!notes_type)
    {
      static const GTypeInfo notes_info =
	{
	  sizeof(RubricaNotesClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_notes_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaNotes),
	  0,
	  (GInstanceInitFunc) rubrica_notes_init
	};

      notes_type = g_type_register_static (G_TYPE_OBJECT, "RubricaNotes",
					   &notes_info, 0);
    }
  
  return notes_type;
}


static void
rubrica_notes_class_init(RubricaNotesClass* klass)
{
  GObjectClass *class;
  GParamSpec* pspec;
  
  class  = G_OBJECT_CLASS (klass);

  class->dispose  = (GObjectFinalizeFunc) rubrica_notes_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_notes_finalize;
  
  class->set_property = rubrica_notes_set_property;
  class->get_property = rubrica_notes_get_property;

  /**
   * RubricaNotes:has-partner:
   * 
   * contact has got a partner
   */
  pspec = g_param_spec_boolean("has-partner", 
			       "has partner", 
			       "contact has got a partner", 
			       FALSE, G_PARAM_READWRITE);
  g_object_class_install_property(class, NOTES_HAS_PARTNER, pspec);

  /**
   * RubricaNotes:partner-name:
   * 
   * partner's name (first name last name)
   */
  pspec = g_param_spec_string("partner-name", 
			      "partner",
			      "partner's name",
			      NULL, G_PARAM_READWRITE);
  g_object_class_install_property(class, NOTES_PARTNER, pspec);

  /**
   * RubricaNotes:children:
   * 
   * notes about children, if any
   */
  pspec = g_param_spec_string("children",
			      "children",
			      "notes about children", 
			      NULL, G_PARAM_READWRITE);
  g_object_class_install_property(class, NOTES_CHILDREN, pspec);

  /**
   * RubricaNotes:notes:
   * 
   * contact's hobbies and other notes
   */
  pspec = g_param_spec_string("other-notes", "other notes", 
			      "contact's hobbies and other notes",
			      NULL, G_PARAM_READWRITE);
  g_object_class_install_property(class, NOTES_OTHER_NOTES, pspec);

  /**
   * RubricaNotes:pubkey:
   * 
   * contact's public key
   */
  pspec = g_param_spec_string("pubkey", "pubkey", "contact's public key", 
			      NULL, G_PARAM_READWRITE);
  g_object_class_install_property(class, NOTES_PUBKEY, pspec);
}


static void
rubrica_notes_init(RubricaNotes* self)
{
  self->private = g_new(RubricaNotesPrivate, 1);
  self->private->dispose_has_run = FALSE;

  self->private->has_partner = FALSE;
  self->private->partner     = NULL;
  self->private->children    = NULL;
  self->private->notes       = NULL;
  self->private->pubkey      = NULL;
  self->private->birthday    = rubrica_date_new();
  self->private->anniversary = rubrica_date_new();
}



static void 
rubrica_notes_dispose (RubricaNotes* self)
{
  g_return_if_fail(IS_RUBRICA_NOTES(self));

  if (self->private->dispose_has_run)
    return;

  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_notes_finalize (RubricaNotes* self)
{
  g_return_if_fail(IS_RUBRICA_NOTES(self));

  rubrica_utils_free_string(self->private->partner);
  rubrica_utils_free_string(self->private->children);
  rubrica_utils_free_string(self->private->notes);
  rubrica_utils_free_string(self->private->pubkey);

  rubrica_date_free(self->private->birthday);
  rubrica_date_free(self->private->anniversary);

  g_free(self->private);
  self->private = NULL;
}


static void 
rubrica_notes_set_property (GObject* obj, guint property_id,
			    const GValue* value, GParamSpec* spec)
{
  RubricaNotes *self = (RubricaNotes *) obj;
  
  switch(property_id)
    {
    case NOTES_HAS_PARTNER:
      self->private->has_partner = g_value_get_boolean(value);
      break;

    case NOTES_PARTNER:
      g_free(self->private->partner);
      self->private->partner = g_value_dup_string(value);
      break;
    
    case NOTES_CHILDREN:
      g_free(self->private->children);
      self->private->children = g_value_dup_string(value);
      break;
    
    case NOTES_OTHER_NOTES:
      g_free(self->private->notes);
      self->private->notes = g_value_dup_string(value);
      break;

    case NOTES_PUBKEY:
      g_free(self->private->pubkey);
      self->private->pubkey = g_value_dup_string(value);
      break;
      
    default:
      break;
    }
}


static void 
rubrica_notes_get_property (GObject* obj, guint property_id,
			    GValue* value, GParamSpec* spec)
{
  RubricaNotes *self = (RubricaNotes *) obj;
  
  switch(property_id)
    {
    case NOTES_HAS_PARTNER:
      g_value_set_boolean (value, self->private->has_partner);
      break;
      
    case NOTES_PARTNER:
      g_value_set_string (value, self->private->partner);
      break;
    
    case NOTES_CHILDREN:
      g_value_set_string (value, self->private->children);
      break;
    
    case NOTES_OTHER_NOTES:
      g_value_set_string (value, self->private->notes);
      break;

    case NOTES_PUBKEY:
      g_value_set_string (value, self->private->pubkey);
     break;
      
    default:
      break;
    }
}


/*  Public
*/

/**
 * rubrica_notes_new
 *
 * create a new #RubricaNotes
 *
 * returns: a RubricaNotes*
 */
RubricaNotes*
rubrica_notes_new(void)
{
  RubricaNotes* notes;

  notes = g_object_new(rubrica_notes_get_type(), NULL);

  return notes;
}


/**
 * rubrica_notes_free
 * @notes: a #RubricaNotes
 *
 * free memory owned by notes
 */
void
rubrica_notes_free(RubricaNotes* notes)
{
  g_return_if_fail(IS_RUBRICA_NOTES(notes));

  g_object_unref(notes);   
}


/**
 * rubrica_notes_set_know_birthday
 * @notes: a #RubricaNotes
 * @know: gboolean
 *
 * set the know birthday flag
 */
void 
rubrica_notes_set_know_birthday (RubricaNotes* notes, gboolean know)
{
  g_return_if_fail(IS_RUBRICA_NOTES(notes));
  
  rubrica_date_know_date(notes->private->birthday, know);;
}


/**
 * rubrica_notes_set_know_anniversary
 * @notes: a #RubricaNotes
 * @know: gboolean
 *
 * set the know anniversary flag
 */
void 
rubrica_notes_set_know_anniversary (RubricaNotes* notes, gboolean know)
{
  g_return_if_fail(IS_RUBRICA_NOTES(notes));

  rubrica_date_know_date(notes->private->anniversary, know);
}



/**
 * rubrica_notes_set_birthday 
 * @notes: a #RubricaNotes
 * @day:  
 * @month: 
 * @year: 
 *
 * set the partner's birthday
 */
void 
rubrica_notes_set_birthday (RubricaNotes* notes, gint day,
			    gint month, gint year)
{
  rubrica_date_set_date(notes->private->birthday, day, month, year);
}


/**
 * rubrica_notes_set_anniversary
 * @notes: a #RubricaNotes
 * @day:  
 * @month: 
 * @year: 
 *
 * set the contact's and his/her partner anniversary
 */
void 
rubrica_notes_set_anniversary (RubricaNotes* notes, gint day,
			       gint month, gint year)
{
  rubrica_date_set_date(notes->private->anniversary, day, month, year);
}


/**
 * rubrica_notes_known_birthday
 * @notes: a #RubricaNotes
 *
 * get the known_birthday flag value
 *
 * returns: gboolean. %TRUE if partner's birthday is known, %FALSE otherwise
 */
gboolean 
rubrica_notes_know_birthday (RubricaNotes* notes)
{
  g_return_val_if_fail(IS_RUBRICA_NOTES(notes), FALSE);

  return rubrica_date_is_date_known(notes->private->birthday);
}


/**
 * rubrica_notes_know_anniversary
 * @notes: a #RubricaNotes
 *
 * get the known_anniversary flag value
 *
 * returns:gboolean. %TRUE if contact and his/her partner's 
 * anniversary is known, %FALSE otherwise
 */
gboolean 
rubrica_notes_know_anniversary (RubricaNotes* notes)
{
  g_return_val_if_fail(IS_RUBRICA_NOTES(notes), FALSE);

  return rubrica_date_is_date_known(notes->private->anniversary);
}


/**
 * rubrica_notes_get_birth_day
 * @notes: a #RubricaNotes
 *
 * get the partner's birth day. Caller must free the returned value
 *
 * returns: a gchar*
 */
gchar*   
rubrica_notes_get_birth_day (RubricaNotes* notes)
{
  gchar* ret;

  g_return_val_if_fail(IS_RUBRICA_NOTES(notes), NULL);
  
  g_object_get(notes->private->birthday, "day", &ret, NULL);

  return ret;
}


/**
 * rubrica_notes_get_birth_month
 * @notes: a #RubricaNotes
 *
 * get the partner's birth month. Caller must free the returned value
 *
 * returns: a gchar*
 */
gchar*   
rubrica_notes_get_birth_month (RubricaNotes* notes)
{
  gchar* ret;

  g_return_val_if_fail(IS_RUBRICA_NOTES(notes), NULL);
 
  g_object_get(notes->private->birthday, "month", &ret, NULL);

  return ret;
}


/**
 * rubrica_notes_get_birth_year
 * @notes: a #RubricaNotes
 *
 * get the partner's birth year. Caller must free the returned value
 *
 * returns: a gchar*
 */
gchar*   
rubrica_notes_get_birth_year (RubricaNotes* notes)
{
  gchar* ret;

  g_return_val_if_fail(IS_RUBRICA_NOTES(notes), NULL);
 
  g_object_get(notes->private->birthday, "year", &ret, NULL);

  return ret;
}


/**
 * rubrica_notes_get_anniversary_day 
 * @notes: a #RubricaNotes
 *
 * get the anniversary day. Caller must free the returned value
 *
 * returns: a gchar*
 */
gchar*   
rubrica_notes_get_anniversary_day (RubricaNotes* notes)
{
  gchar* ret;

  g_return_val_if_fail(IS_RUBRICA_NOTES(notes), NULL);
  
  g_object_get(notes->private->anniversary, "year", &ret, NULL);

  return ret;
}


/**
 * rubrica_notes_get_anniversary_month
 * @notes: a #RubricaNotes
 *
 * get the anniversary month. Caller must free the returned value
 *
 * returns: a gchar*
 */
gchar*   
rubrica_notes_get_anniversary_month (RubricaNotes* notes)
{
  gchar* ret;

  g_return_val_if_fail(IS_RUBRICA_NOTES(notes), NULL);
 
  g_object_get(notes->private->anniversary, "month", &ret, NULL);
  
  return ret;
}


/**
 * rubrica_notes_get_anniversary_year
 * @notes: a #RubricaNotes
 *
 * get the anniversary year. Caller must free the returned value
 *
 * returns: a gchar*
 */
gchar*   
rubrica_notes_get_anniversary_year (RubricaNotes* notes)
{
  gchar* ret;

  g_return_val_if_fail(IS_RUBRICA_NOTES(notes), NULL);
 
  g_object_get(notes->private->anniversary, "year", &ret, NULL);

  return ret;
}


/**
 * rubrica_notes_get_birthdate
 * @notes: a #RubricaNotes
 *
 * get partner's birthday, or unknown. Caller must free the returned value
 *
 * returns: a gchar*
 */
gchar*
rubrica_notes_get_birthdate (RubricaNotes* notes)
{
  g_return_val_if_fail(IS_RUBRICA_NOTES(notes), g_strdup(_("unknown")));
  
  return rubrica_date_get_human_date(notes->private->birthday);
}


/**
 * rubrica_notes_get_anniversarydate
 * @notes: a #RubricaNotes
 *
 * get contact and partner's anniversary, or unknown. 
 * Caller must free the returned value
 *
 * returns: a gchar*
 */
gchar*
rubrica_notes_get_anniversarydate (RubricaNotes* notes)
{
  g_return_val_if_fail(IS_RUBRICA_NOTES(notes), g_strdup(_("unknown")));

  return rubrica_date_get_human_date(notes->private->anniversary);
}


/**
 * rubrica_notes_append_other_notes
 * @notes: a #RubricaNotes
 * @other_note:
 *
 * append to other note 
 */
void rubrica_notes_append_other_notes (RubricaNotes* notes, gchar* other_note)
{
  gchar* tmp = NULL;
  gchar* new;

  g_return_if_fail(IS_RUBRICA_NOTES(notes));
  g_return_if_fail(other_note != NULL);
  
  g_object_get(notes, "other-notes", &tmp, NULL);

  if (tmp && (g_ascii_strcasecmp(tmp, "") != 0))
    new = g_strdup_printf("%s\n%s", other_note, tmp);
  else
    new = g_strdup_printf("%s\n", other_note);

  g_object_set(notes, "other-notes", tmp, NULL);
}


/**
 * rubrica_notes_copy
 * @notes: a #RubricaNotes
 *
 * copy the given notes
 *
 * returns: a new allocated #RubricaNotes* 
 */
RubricaNotes* 
rubrica_notes_copy (RubricaNotes* notes)
{
  RubricaNotes* new;
  gchar *partner, *children, *other, *pubkey;
  gboolean has_partner;

  g_return_val_if_fail(IS_RUBRICA_NOTES(notes), NULL);

  new = rubrica_notes_new();

  g_object_get(notes, "has-partner", &has_partner, "partner-name", &partner, 
	       "children", &children, "other-notes", &other, 
	       "pubkey", &pubkey, NULL);
  
  g_object_set(new, "has-partner", has_partner, "partner-name", partner, 
	       "children", children, "other-notes", other,
	       "pubkey", pubkey, NULL);
	
  rubrica_notes_set_know_birthday(new, rubrica_notes_know_birthday(notes));
  rubrica_notes_set_know_anniversary(new, 
				     rubrica_notes_know_anniversary(notes));
  
  rubrica_notes_set_birthday (new, 
			      atoi(rubrica_notes_get_birth_day(notes)),
			      atoi(rubrica_notes_get_birth_month(notes)),
			      atoi(rubrica_notes_get_birth_year(notes)));

  rubrica_notes_set_anniversary(new, 
			    atoi(rubrica_notes_get_anniversary_day(notes)),
			    atoi(rubrica_notes_get_anniversary_month(notes)),
			    atoi(rubrica_notes_get_anniversary_year(notes)));

  return new;
}
