/*
*  RAL -- Rubrica Addressbook Library
*  file: personal.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <time.h>
#include <glib.h>
#include <glib-object.h>

#include "types.h"
#include "card.h"
#include "personal.h"
#include "contact.h"
#include "work.h"
#include "notes.h"
#include "utils.h"
#include "save.h"
#include "load.h"


struct _RubricaPersonalCardPrivate {

  RubricaContact* contact;
  RubricaWork*    work;
  RubricaNotes*   notes;

  gboolean dispose_has_run;
};



static void rubrica_personal_card_class_init (RubricaPersonalCardClass* klass);
static void rubrica_personal_card_init       (RubricaPersonalCard* obj);

static void rubrica_personal_card_dispose    (RubricaPersonalCard* obj);
static void rubrica_personal_card_finalize   (RubricaPersonalCard* obj);


static void rubrica_personal_card_free       (RubricaPersonalCard* self);

RubricaPersonalCard* rubrica_personal_card_copy (RubricaPersonalCard* self);
RubricaInfos* rubrica_personal_card_get_infos (RubricaPersonalCard* self);


static RubricaCardClass* parent_class;


GType
rubrica_personal_card_get_type()
{
  static GType personal_card_type = 0;
  
  if (!personal_card_type)
    {
      static const GTypeInfo personal_card_info =
	{
	  sizeof(RubricaPersonalCardClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_personal_card_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaPersonalCard),
	  0,
	  (GInstanceInitFunc) rubrica_personal_card_init
	};
      
      personal_card_type = g_type_register_static (RUBRICA_CARD_TYPE,
						   "RubricaPersonalCard",
						   &personal_card_info, 0);
    }
  
  return personal_card_type;
}


static void
rubrica_personal_card_class_init(RubricaPersonalCardClass* klass)
{
  GObjectClass *obj_class;
  
  parent_class                  = RUBRICA_CARD_CLASS(klass);
  parent_class->free            = (gpointer) rubrica_personal_card_free;
  parent_class->copy            = (gpointer) rubrica_personal_card_copy;

  parent_class->write_infos     = (gpointer) rubrica_save_write_infos;
  parent_class->write_contact   = (gpointer) rubrica_save_write_contact;
  parent_class->write_group     = (gpointer) rubrica_save_write_group;
  parent_class->write_work      = (gpointer) rubrica_save_write_work;
  parent_class->write_addresses = (gpointer) rubrica_save_write_addresses;
  parent_class->write_net       = (gpointer) rubrica_save_write_net;
  parent_class->write_telephone = (gpointer) rubrica_save_write_telephone;
  parent_class->write_notes     = (gpointer) rubrica_save_write_notes;

  parent_class->read_infos      = (gpointer) rubrica_load_read_infos;
  parent_class->read_contact    = (gpointer) rubrica_load_read_contact;
  parent_class->read_group      = (gpointer) rubrica_load_read_groups;
  parent_class->read_work       = (gpointer) rubrica_load_read_work;
  parent_class->read_addresses  = (gpointer) rubrica_load_read_addresses;
  parent_class->read_net        = (gpointer) rubrica_load_read_net;
  parent_class->read_telephone  = (gpointer) rubrica_load_read_telephone;
  parent_class->read_notes      = (gpointer) rubrica_load_read_notes;

  parent_class->get_infos       = (gpointer) rubrica_personal_card_get_infos;
  parent_class->print_card      = NULL;

  obj_class = G_OBJECT_CLASS (klass);
  obj_class->dispose  = (GObjectFinalizeFunc) rubrica_personal_card_dispose;
  obj_class->finalize = (GObjectFinalizeFunc) rubrica_personal_card_finalize;
}


static void
rubrica_personal_card_init(RubricaPersonalCard* self)
{
  self->private = g_new(RubricaPersonalCardPrivate, 1);

  self->private->contact         = NULL;
  self->private->work            = NULL;
  self->private->notes           = NULL;

  self->private->dispose_has_run = FALSE;  
}



static void 
rubrica_personal_card_dispose (RubricaPersonalCard* self)
{
  g_return_if_fail(IS_RUBRICA_PERSONAL_CARD(self));
  
  if (self->private->dispose_has_run)
    return;
  
  /*    free the objects owned by personal card
   */
  rubrica_contact_free(RUBRICA_CONTACT(self->private->contact));
  rubrica_work_free(RUBRICA_WORK(self->private->work));
  rubrica_notes_free(RUBRICA_NOTES(self->private->notes));  

  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_personal_card_finalize (RubricaPersonalCard* self)
{
  g_return_if_fail(IS_RUBRICA_PERSONAL_CARD(self));
  
  /*    finally, delete personal card's private structure
   */
  g_free(self->private);
  self->private = NULL;
}


static void
rubrica_personal_card_free(RubricaPersonalCard* self)
{
  g_return_if_fail(IS_RUBRICA_PERSONAL_CARD(self));

  g_object_unref(G_OBJECT(self));
}



RubricaInfos* 
rubrica_personal_card_get_infos (RubricaPersonalCard* card)
{
  RubricaInfos* infos;
  RubricaContact *contact = NULL;
  RubricaWork    *work    = NULL;
  
  infos = g_malloc0(sizeof(RubricaInfos));
  if (!infos)
    g_error("Unable to get enougth memory");

  contact = rubrica_personal_card_get_contact(card);
  work    = rubrica_personal_card_get_work(card);

  if (contact)
    g_object_get(contact, "first-name", &infos->first, 
		 "last-name", &infos->last, 
		 "profession", &infos->prof, NULL);
  else
    infos->first = infos->last = infos->prof = g_strdup(NULL);
  
  if (work)
    g_object_get(work, "assignment", &infos->assignment, NULL);
  else
    infos->assignment = g_strdup(NULL);

  return infos;
}



/*   Public
 */

/**
 * rubrica_personal_card_new
 *
 * create a new #RubricaPersonalCard
 *
 * returns: a RubricaPersonalCard*
 */
RubricaPersonalCard*
rubrica_personal_card_new(void)
{
  RubricaPersonalCard* card;
  
  gulong id = 0L;
  gchar* pc_id = NULL;

  card = g_object_new(rubrica_personal_card_get_type(), NULL);

  id = RUBRICA_CARD_CLASS(parent_class)->generate_id(RUBRICA_CARD(card), 
						     "personal");
  if (id == 0L)
    return NULL;

  pc_id = g_strdup_printf("pc%ld", id); 
  rubrica_card_set_id(RUBRICA_CARD(card), pc_id); 
  
  g_free(pc_id);
  pc_id = NULL;
  
  return card;
}

/**
 * rubrica_personal_card_set_contact
 * @card: a #RubricaPersonalCard
 * @contact: a #RubricaContact
 *
 * store contact's data in the card
 */
void 
rubrica_personal_card_set_contact (RubricaPersonalCard* card, 
				   RubricaContact* contact)
{
  g_return_if_fail(IS_RUBRICA_PERSONAL_CARD(card));
  g_return_if_fail(IS_RUBRICA_CONTACT(contact));

  if (card->private->contact)
    rubrica_contact_free(RUBRICA_CONTACT(card->private->contact));

  card->private->contact = contact;
}


/**
 * rubrica_personal_card_set_work
 * @card: a #RubricaPersonalCard
 * @work: a #RubricaWork
 *
 * store contact's work data in card
 */
void 
rubrica_personal_card_set_work (RubricaPersonalCard* card,
				RubricaWork* work)
{
  g_return_if_fail(IS_RUBRICA_PERSONAL_CARD(card));
  g_return_if_fail(IS_RUBRICA_WORK(work));

  if (card->private->work)
    rubrica_work_free(RUBRICA_WORK(card->private->work));

  card->private->work = work;
}

/**
 * rubrica_personal_card_set_notes
 * @card: a #RubricaPersonalCard
 * @notes: a #RubricaNotes
 *
 * store contact's notes in card
 */
void 
rubrica_personal_card_set_notes (RubricaPersonalCard* card, 
				 RubricaNotes* notes)
{
  g_return_if_fail(IS_RUBRICA_PERSONAL_CARD(card));
  g_return_if_fail(IS_RUBRICA_NOTES(notes));

  if (card->private->notes)
    rubrica_notes_free(RUBRICA_NOTES(card->private->notes));

  card->private->notes = notes;
}



/**
 * rubrica_personal_card_get_contact
 * @card: a #RubricaPersonalCard
 *
 * get the contact's personal data from card
 *
 * returns: a RubricaContact* or NULL
 */
RubricaContact*      
rubrica_personal_card_get_contact (RubricaPersonalCard* card)
{  
  g_return_val_if_fail(IS_RUBRICA_PERSONAL_CARD(card), NULL);

  return card->private->contact;
}


/**
 * rubrica_personal_card_get_work
 * @card: a #RubricaPersonalCard
 *
 * get the contact work's data from card
 *
 * returns: a RubricaWork* or NULL
 */
RubricaWork*       
rubrica_personal_card_get_work (RubricaPersonalCard* card)
{
  g_return_val_if_fail(IS_RUBRICA_PERSONAL_CARD(card), NULL);

  return card->private->work;
}


/**
 * rubrica_personal_card_get_notes
 * @card: a #RubricaPersonalCard
 *
 * get the contact's notes from card
 *
 * returns: a RubricaNotes* or NULL
 */ 
RubricaNotes*      
rubrica_personal_card_get_notes (RubricaPersonalCard* card)
{
  g_return_val_if_fail(IS_RUBRICA_PERSONAL_CARD(card), NULL);

  return card->private->notes;
}


/**
 * rubrica_personal_card_copy
 * @card: a #RubricaPersonalCard
 *
 * make a copy of given card
 *
 * returns: a new #RubricaPersonalCard* or NULL
 */
RubricaPersonalCard*  
rubrica_personal_card_copy (RubricaPersonalCard* card) 
{
  RubricaPersonalCard* new;
  RubricaContact* contact = NULL;
  RubricaWork* work = NULL;
  RubricaNotes* notes = NULL;
  
  g_return_val_if_fail(IS_RUBRICA_PERSONAL_CARD(card), NULL);

  new = rubrica_personal_card_new();

  contact = rubrica_contact_copy(rubrica_personal_card_get_contact(card));
  work    = rubrica_work_copy(rubrica_personal_card_get_work(card));
  notes   = rubrica_notes_copy(rubrica_personal_card_get_notes(card));
  
  rubrica_personal_card_set_contact(new, contact);
  rubrica_personal_card_set_work(new, work);
  rubrica_personal_card_set_notes(new, notes);

  return new;
}



