/*
*  RAL -- Rubrica Addressbook Library
*  file: ref.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <glib.h>
#include <glib-object.h>

#include "ref.h"
#include "utils.h"


enum { 
  RUBRICA_REF_ID = 1,
  RUBRICA_REF_NAME,
};



struct _RubricaRefPrivate {
  gchar* id;          /* card id     */  /* reference to other cards, that */
  gchar* info;        /* card's info */  /* are in relation whith this one */

  gboolean dispose_has_run;
};



static void rubrica_ref_class_init (RubricaRefClass* klass);
static void rubrica_ref_init       (RubricaRef* obj);

static void rubrica_ref_dispose    (RubricaRef* obj);
static void rubrica_ref_finalize   (RubricaRef* obj);


static void rubrica_ref_set_property (GObject* obj, guint property_id,
				      const GValue* value, GParamSpec* spec);
static void rubrica_ref_get_property (GObject* obj, guint property_id,
				      GValue* value, GParamSpec* spec);




GType
rubrica_ref_get_type()
{
  static GType rubrica_ref_type = 0;
  
  if (!rubrica_ref_type)
    {
      static const GTypeInfo rubrica_ref_info =
	{
	  sizeof(RubricaRefClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_ref_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaRef),
	  0,
	  (GInstanceInitFunc) rubrica_ref_init
	};

      rubrica_ref_type = g_type_register_static (G_TYPE_OBJECT, 
						 "RubricaRef",
						 &rubrica_ref_info, 0);
    }
  
  return rubrica_ref_type;
}


static void
rubrica_ref_class_init(RubricaRefClass* klass)
{
  GObjectClass *parent_class;
  GParamSpec* pspec;
  
  parent_class  = G_OBJECT_CLASS (klass);

  parent_class->dispose      = (GObjectFinalizeFunc) rubrica_ref_dispose;
  parent_class->finalize     = (GObjectFinalizeFunc) rubrica_ref_finalize;

  parent_class->set_property = rubrica_ref_set_property;
  parent_class->get_property = rubrica_ref_get_property;

  /* class property 
   */
  /**
   * RubricaRef:id
   *
   * the id of the related card
   */
  pspec = g_param_spec_string("id", 
			      "card id", 
			      "the id of the related card",  
			      NULL,    
			      G_PARAM_READWRITE); 
  g_object_class_install_property(parent_class, RUBRICA_REF_ID, pspec);  
  
  /**
   * RubricaRef:ref-info
   *
   * the infos related to the card
   */
  pspec = g_param_spec_string("ref-info", 
			      "card infos", 
			      "the infos related to the card", 
			      NULL,  
			      G_PARAM_READWRITE);  
  g_object_class_install_property(parent_class, RUBRICA_REF_NAME, pspec);  
}


static void
rubrica_ref_init(RubricaRef* self)
{
  self->private = g_new(RubricaRefPrivate, 1);

  self->private->id   = NULL;
  self->private->info = NULL;

  self->private->dispose_has_run = FALSE;
}

 

static void 
rubrica_ref_set_property (GObject* obj, guint property_id,
			  const GValue* value, GParamSpec* spec)
{
  RubricaRef* self = (RubricaRef*) obj;

  switch (property_id) 
    {
    case RUBRICA_REF_ID:
      g_free(self->private->id);
      self->private->id = g_value_dup_string(value); 
      break; 

    case RUBRICA_REF_NAME:
      g_free(self->private->info);
      self->private->info = g_value_dup_string(value); 
      break; 
      
    default: 
      break; 
    } 
} 
 

static void 
rubrica_ref_get_property (GObject* obj, guint property_id,
			  GValue* value, GParamSpec* spec)
{
  RubricaRef* self = (RubricaRef*) obj;

  switch (property_id) 
    {
    case RUBRICA_REF_ID: 
      g_value_set_string(value, self->private->id); 
      break; 
      
    case RUBRICA_REF_NAME:
      g_value_set_string(value, self->private->info); 
      break; 
      
    default:
      break;  
    }  
}


static void 
rubrica_ref_dispose (RubricaRef* self)
{
  g_return_if_fail(IS_RUBRICA_REF(self));
  
  if (self->private->dispose_has_run)
    return;

  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_ref_finalize (RubricaRef* self)
{
  g_return_if_fail(IS_RUBRICA_REF(self));
  
  rubrica_utils_free_string(self->private->id);
  rubrica_utils_free_string(self->private->info);
  
  g_free(self->private);
  self->private = NULL;
}



/*   Public
*/
/**
 * rubrica_ref_new
 *
 * create a #RubricaRef
 *
 * returns: a new #RubricaRef*
 */
RubricaRef*
rubrica_ref_new(gchar* id)
{
  RubricaRef* ref;

  ref = g_object_new(rubrica_ref_get_type(), "id", id, NULL);
  
  return ref;
}

/**
 * rubrica_ref_with_id_new
 * @id: a unique id. Id is get from a #RubricaCard.
 * 
 * create a #RubricaRef an initialize to id
 *
 * returns: a new #RubricaRef*
 *//*
RubricaRef*
rubrica_ref_with_id_new(gchar* id)
{
  RubricaRef* ref;

  ref = g_object_new(rubrica_ref_get_type(), "id", id, NULL);

  return ref;
}
 */

/**
 * rubrica_ref_free
 * @ref: a #RubricaRef
 *
 * free the #RubricaRef
 */
void
rubrica_ref_free(RubricaRef* ref)
{
  g_return_if_fail(IS_RUBRICA_REF(ref));

  g_object_unref(ref);   
}



/**
 * rubrica_ref_copy
 * @ref:
 *
 * copy the given object
 *
 * returns: a new allocated #RubricaRef* 
 */
RubricaRef* 
rubrica_ref_copy (RubricaRef* ref)
{
  RubricaRef* new;
  gchar* id;
  gchar* info;

  g_return_val_if_fail(IS_RUBRICA_REF(ref), NULL);

  g_object_get(G_OBJECT(ref), "id", &id, "ref-info", &info, NULL);

  new = rubrica_ref_new(id);
  g_object_set(G_OBJECT(new), "ref-info", info, NULL);

  return new;
}
