/*
*  RAL -- Rubrica Addressbook Library
*  file: save.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <glib.h>
#include <libxml/tree.h>

#include "save.h"
#include "card.h"
#include "personal.h"
#include "company.h"
#include "contact.h"
#include "address.h"
#include "group.h"
#include "net.h"
#include "notes.h"
#include "telephone.h"
#include "ref.h"
#include "io.h"
#include "error.h"


/*
  RubricaDoc file structure:
     version:    rubrica's version that made the addressbook file
     fileformat: #4
     doctype:    addressbook

  <?xml version="1.0" encoding="UTF-8"?>
  <Rubrica version="" fileformat="" doctype="">        root
     <Card                                             root child 
            name=""                                    the card name
            deletable=""                               is a deletable card?
            deleted=""                                 was card deleted?
            rank=""                                    card rank (1..5)
            id="pcxxx"                                 card id   (es:  pc643)
	    created=""                                 creation date
            last_change=""                             last change date
	    type="">                                   card's type 
         <Groups>                                      card child 
             <Group owner="" pixmap="">....</Group>    groups children
	     ...
         </Groups>       
         <Data>                                           card child
             <FirstName>..................</FirstName>    data child 
	     <MiddleName>.................</MiddleName>     "    "
	     <LastName>...................</LastName>                    
	     <Nickname>...................</NickName>                    
	     <Profession>.................</Profession>                  
 	     <NamePrefix>.................</NamePrefix>                  
	     <Title>......................</Title>                       
	     <Photo type="">..............</Photo>
	     <Birthday known="" day="" month="" year=""/>  
         </Data>
	 <Work>
	     <Assignment>.................</Assignment>
	     <Organization>...............</Organization>
             <Department>.................</Department>
	     <SubDepartment>..............</SubDepartment>
	     <ManagerName>................</ManagerName>
	     <ManagerPhone>...............</ManagerPhone>
	     <CollabratorName>............</CollabratorName>
	     <CollaboratorPhone>..........</CollabratorPhone>
         </Work>
	 <Addresses>                                   card child
             <Address type="">                         addresses children
                 <Street number="">.......</Street>    address child
	         <City zip="">............</City>
	         <Province>...............</Province>
	         <State>..................</State>
	         <Country>................</Country>
 	      </Address>
	     ...
         </Addresses>
	 <Net>
	     <Uri type="web">.............</Uri>
	     <Uri type="email">...........</Uri>
	     <Uri type="aim">.............</Uri>
	     <Uri type="msn">.............</Uri>
	     <Uri type="yahoo">...........</Uri>
	     <Uri type="icq">.............</Uri>
	     <Uri type="jabber">..........</Uri>	     
	     ...
	 </Net>
	 <TelephoneNumbers>
	     <Telephone type="home">......</Telephone>
	     <Telephone type="work">......</Telephone>
	     <Telephone type="fax">.......</Telephone>
	     ...
 	 </TelephoneNumbers>
	 <Notes married="">
             <SpouseName>.................</SpouseName>
	     <Child>......................</Child>
	     <OtherNotes>.................</OtherNotes>
	     <PublicKey>..................</PublicKey>
	     <SpouseBirthday known="" day="" month="" year=""/>
	     <Anniversary known="" day="" month="" year""/>
         </Notes>
     </Card>
     <Card>
     <Card                                              root child 
            name=""                                     the card name
            deletable=""                                is a deletable card?
            deleted=""                                  was card deleted?
            rank=""                                     card rank (1..5)
            id="ccxxx"                                  card id   (es: cc1039)
	    created=""                                  creation date
            last_change=""                              last change date
	    type="company">                             card's type 
         <Groups>                                       card child 
             <Group>......................</Group>      groups children
	     ...
         </Groups>       
         <Company>
             <CompanyName>.............</CompanyName>
	     <Pi>......................</Pi>
	     <Notes>...................</Notes>
	     <Refs>
	        <Ref refid="">.........</Ref>
		....
	     </Refs>
         </Company>
	 <CompanyContacts>
	     <Contact ref_id="id_personal">Nome Cognome</Contact>
	     ...
	 <CompanyContacts>
	 <Addresses>                                    card child
             <Address type="">                          addresses children
                 <Street number="">.......</Street>     address child
	         <City zip="">............</City>
	         <Province>...............</Province>
	         <State>..................</State>
	         <Country>................</Country>
 	      </Address>
	     ...
         </Addresses>
	 <Net>
	     <Uri type="web">..........</Uri>
	     <Uri type="email">........</Uri> 
	     ...
	 </Net>
	 <TelephoneNumbers>
	     <Telephone type="green">......</Telephone>
	     <Telephone type="operator">...</Telephone>
	     <Telephone type="fax">........</Telephone>
	     ...
 	 </TelephoneNumbers>
     </Card>
     ...
  </Rubrica>


  Rubrica
     |
     | children
     |
     v   next         next         next 
   Card-------->Card-------->....------->Card
     |
     | children
     |
     v   next                
   Groups--->Data--->Addresses--->Net--->....Phone-->Work-->Notes
     |        \          |         |           |        |      |      
     | child  | child    | child   |           |        |      |     
     |        |          v         v           v        v      v     
     |        |         ...       ...         ...      ...     ...    ...   
     v        v
   Group    FirstName--->MiddleName--->...

   pcxxx => personal card xxx
   ccxxx => company  card xxx
*/



/**
 * rubrica_save_write_card
 * @card: a #RubricaCard
 * @cardxml: a #xmlNodePtr
 *
 * Write the #RubricaCard into file. This function calls the
 * virtual functions defined in #RubricaPersonalCard and #RubricaCompanyCard.
 */
void
rubrica_save_write_card(RubricaCard *card, xmlNodePtr cardxml)
{
  RubricaCardClass* klass;

  klass = RUBRICA_CARD_GET_CLASS(card);

  /*    Card infos	
  */
  if (RUBRICA_CARD_CLASS(klass)->write_infos)  
    RUBRICA_CARD_CLASS(klass)->write_infos(card, cardxml);
  
  /*    Groups
   */
  if (RUBRICA_CARD_CLASS(klass)->write_group)  
    RUBRICA_CARD_CLASS(klass)->write_group(card, cardxml);

  /*    Personal (Contact or Company) data
  */
  if (RUBRICA_CARD_CLASS(klass)->write_contact)  
    RUBRICA_CARD_CLASS(klass)->write_contact(card, cardxml);
  
  /*    Work data
  */
  if (RUBRICA_CARD_CLASS(klass)->write_work)  
    RUBRICA_CARD_CLASS(klass)->write_work(card, cardxml);
  
  /*    Addresses
   */
  if (RUBRICA_CARD_CLASS(klass)->write_addresses)  
    RUBRICA_CARD_CLASS(klass)->write_addresses(card, cardxml);

  /*    NetAddresses
   */
  if (RUBRICA_CARD_CLASS(klass)->write_net)  
    RUBRICA_CARD_CLASS(klass)->write_net(card, cardxml);

  /*    Telephone numbers
  */
  if (RUBRICA_CARD_CLASS(klass)->write_telephone)  
    RUBRICA_CARD_CLASS(klass)->write_telephone(card, cardxml);

  /*    Notes
   */
  if (RUBRICA_CARD_CLASS(klass)->write_notes)  
    RUBRICA_CARD_CLASS(klass)->write_notes(card, cardxml);  
}


/*  Card Infos 
*/
void 
rubrica_save_write_infos(RubricaCard *card, xmlNodePtr cardxml)
{
  gchar *name, *type;
  gboolean deletable, deleted;
  RubricaRank rank;

  g_object_get(card, "card-name", &name, "card-type", &type,
	       "deletable", &deletable, "deleted", &deleted, 
	       "rank", &rank, NULL);

  /* setto le proprietà della scheda 
     nome, editabile, concellabile
  */
  rubrica_io_set_str (cardxml, "id", rubrica_card_get_id(card));
  rubrica_io_set_str (cardxml, "type", type);
  rubrica_io_set_str (cardxml, "name", name);
  rubrica_io_set_bool(cardxml, "deletable", deletable);
  rubrica_io_set_bool(cardxml, "deleted", deleted); 
  rubrica_io_set_num (cardxml, "rank", rank);
  rubrica_io_set_time(cardxml, "created", 
		      rubrica_card_get_creation_time(card));
  rubrica_io_set_time(cardxml, "last_change", 
		      rubrica_card_get_modification_time(card));
  
  g_free(name);
  g_free(type);
}


/* Groups
 */
void
rubrica_save_write_group (RubricaPersonalCard *card,  xmlNodePtr xmlnode)
{
  gpointer data;
  xmlNodePtr groupxml;    /* group node  */

  g_return_if_fail(IS_RUBRICA_CARD(card));

  groupxml = xmlNewTextChild(xmlnode, NULL, (xmlChar*) "Groups", NULL);
  data = rubrica_card_get_group(RUBRICA_CARD(card));
  for (; data; data = rubrica_card_get_next_group(RUBRICA_CARD(card)))
    {
      xmlNodePtr grxml;
      xmlChar* name;
      xmlChar* owner;
      xmlChar* pixmap;

      g_object_get(RUBRICA_GROUP(data), "group-name", &name, 
		   "group-owner", &owner, "group-pixmap", &pixmap, NULL);

      grxml = xmlNewTextChild(groupxml, NULL, (xmlChar*) "Group", name);
      rubrica_io_set_str(grxml, "owner",  (gchar*) owner);
      rubrica_io_set_str(grxml, "pixmap", (gchar*) pixmap);
    }
}	


/*    Personal data
 */
void
rubrica_save_write_contact (RubricaPersonalCard *card, xmlNodePtr xmlnode)
{
  gpointer data;

  g_return_if_fail(IS_RUBRICA_CARD(card));

  data = rubrica_personal_card_get_contact(RUBRICA_PERSONAL_CARD(card));
  if (IS_RUBRICA_CONTACT(data))
    {
      xmlNodePtr contact;
      xmlNodePtr node;  
      gboolean bool;
      xmlChar *first, *middle, *last, *nick;
      xmlChar *prof, *prefix, *title, *photo;
      gchar *day, *month, *year; 
      
      g_object_get(RUBRICA_CONTACT(data), "first-name", &first, 
		   "middle-name", &middle, "last-name", &last, 
		   "nick-name", &nick, "profession", &prof, 
		   "prefix", &prefix, "title", &title, "photo", &photo, NULL);

      day   = rubrica_contact_get_birth_day   (RUBRICA_CONTACT(data));
      month = rubrica_contact_get_birth_month (RUBRICA_CONTACT(data));
      year  = rubrica_contact_get_birth_year  (RUBRICA_CONTACT(data));

      contact = xmlNewTextChild(xmlnode, NULL, (xmlChar*) "Data", NULL); 

      xmlNewTextChild(contact, NULL, (xmlChar*) "FirstName",  first);
      xmlNewTextChild(contact, NULL, (xmlChar*) "MiddleName", middle);
      xmlNewTextChild(contact, NULL, (xmlChar*) "LastName",   last);
      xmlNewTextChild(contact, NULL, (xmlChar*) "NickName",   nick);
      xmlNewTextChild(contact, NULL, (xmlChar*) "Profession", prof);
      xmlNewTextChild(contact, NULL, (xmlChar*) "NamePrefix", prefix);
      xmlNewTextChild(contact, NULL, (xmlChar*) "Title",      title);

      node = xmlNewTextChild(contact, NULL, (xmlChar*) "Photo", photo);
      rubrica_io_set_str(node, "type", "url");
      
      node = xmlNewTextChild(contact, NULL, (xmlChar*) "Birthday", NULL);
      bool = (g_ascii_strcasecmp(day, "BadDay") == 0);
      rubrica_io_set_bool(node, "known", bool);     
      rubrica_io_set_str (node, "day",   day);
      rubrica_io_set_str (node, "month", month);
      rubrica_io_set_str (node, "year",  year); 
    }
}


/*    Addresses
 */
void
rubrica_save_write_addresses (RubricaPersonalCard *card,  xmlNodePtr xmlnode)
{
  gpointer data;
  xmlNodePtr addressxml;  /* addresses node, address's father */ 

  g_return_if_fail(IS_RUBRICA_CARD(card));

  addressxml = xmlNewTextChild(xmlnode, NULL, (xmlChar*) "Addresses", NULL);
  data = rubrica_card_get_address(RUBRICA_CARD(card));
  for (; data; data = rubrica_card_get_next_address(RUBRICA_CARD(card)))
    {
      if (IS_RUBRICA_ADDRESS(data))
	{
	  xmlNodePtr adsxml;      /* address node, addresses's child  */
	  xmlNodePtr node;        /* address's child  */
	  RubricaAddressType adtype = RUBRICA_ADDRESS_UNKNOWN;
	  gchar *type, *street, *number, *city, *zip;
	  gchar *province, *state, *country;

	  /*
	  rubrica_address_get(RUBRICA_ADDRESS(data), &type, &street, &number, 
			      &city, &zip, &province, &state, &country);
	  */
	  g_object_get(RUBRICA_ADDRESS(data), "address-type", &adtype,  
		       "street", &street, "street-number", &number,  
		       "city", &city, "zip", &zip, "province", &province, 
		       "state", &state, "country", &country, NULL);
	  
	  if ((adtype < RUBRICA_ADDRESS_PREF) || 
	      (adtype > RUBRICA_ADDRESS_INVALID))
	    adtype = RUBRICA_ADDRESS_UNKNOWN;

	  type = rubrica_address_decode_type(adtype);
	  
	  adsxml = xmlNewTextChild(addressxml, NULL, 
				   (xmlChar*) "Address", NULL);

	  rubrica_io_set_str(adsxml, "type", type);

	  node = xmlNewTextChild(adsxml, NULL, (xmlChar*) "Street",
				 (xmlChar*) street);
	  rubrica_io_set_str(node, "number", number);

	  node = xmlNewTextChild(adsxml, NULL, (xmlChar*) "City", 
				 (xmlChar*) city);
	  rubrica_io_set_str(node, "zip", zip);

	  xmlNewTextChild(adsxml, NULL, (xmlChar*) "Province", 
			  (xmlChar*) province);
	  xmlNewTextChild(adsxml, NULL, (xmlChar*) "State",   
			  (xmlChar*) state);
	  xmlNewTextChild(adsxml, NULL, (xmlChar*) "Country", 
			  (xmlChar*) country);
	}
    }
}


/*      NetAddresses
 */
void
rubrica_save_write_net (RubricaPersonalCard *card, xmlNodePtr xmlnode)
{
  gpointer data;
  xmlNodePtr netxml;

  g_return_if_fail(IS_RUBRICA_CARD(card));

  netxml = xmlNewTextChild(xmlnode, NULL, (xmlChar*) "Net", NULL);
  data = rubrica_card_get_net_address(RUBRICA_CARD(card));
  for (; data; data = rubrica_card_get_next_net_address(RUBRICA_CARD(card)))
    if (IS_RUBRICA_NET_ADDRESS(data))
      {
	RubricaNetAddressType nettype;
	xmlNodePtr uri;
	gchar *url, *type; 

	g_object_get(RUBRICA_NET_ADDRESS(data), "url", &url, 
		     "url-type", &nettype, NULL);

	uri = xmlNewTextChild(netxml, NULL, (xmlChar*) "Uri", (xmlChar*) url);
	type = rubrica_net_address_decode_type(nettype);
	rubrica_io_set_str(uri, "type", type);
      }
}  


/*      Telephone numbers
 */
void
rubrica_save_write_telephone (RubricaPersonalCard *card, xmlNodePtr xmlnode)
{
  gpointer data;
  xmlNodePtr telxml;

  g_return_if_fail(IS_RUBRICA_CARD(card));

  telxml = xmlNewTextChild(xmlnode, NULL, (xmlChar*) "TelephoneNumbers", NULL);
  data = rubrica_card_get_telephone(RUBRICA_CARD(card));
  for (; data; data = rubrica_card_get_next_telephone(RUBRICA_CARD(card)))
    if (IS_RUBRICA_TELEPHONE(data))
      {
	xmlNodePtr tel;
	gchar *number, *type; 
	RubricaTelephoneType ttype;
	
	g_object_get(RUBRICA_TELEPHONE(data), "telephone-number", &number, 
		     "telephone-type", &ttype, NULL);

	tel = xmlNewTextChild(telxml, NULL,
			      (xmlChar*) "Telephone", (xmlChar*) number);
	type = rubrica_telephone_decode_type(ttype);

	rubrica_io_set_str(tel, "type", type);
      }
}


/*    Work data
 */
void
rubrica_save_write_work (RubricaPersonalCard *card, xmlNodePtr xmlnode)
{
  gpointer data;

  g_return_if_fail(IS_RUBRICA_CARD(card));

  data = rubrica_personal_card_get_work(RUBRICA_PERSONAL_CARD(card));
  if (IS_RUBRICA_WORK(data))
    {     
      xmlNodePtr work;
      xmlChar *assignment, *org, *dep, *subdep;
      xmlChar *manager, *mphone, *collaborator, *cphone;

      g_object_get(RUBRICA_WORK(data), "assignment", &assignment, 
		   "organization", &org, "department", &dep, 
		   "sub-department", &subdep, 
		   "manager-name", &manager, "manager-phone", &mphone, 
		   "collaborator", &collaborator, 
		   "collaborator-phone", &cphone, NULL);

      work = xmlNewTextChild(xmlnode, NULL, (xmlChar*) "Work", NULL);
       
      xmlNewTextChild(work, NULL, (xmlChar*) "Assignment",       assignment);
      xmlNewTextChild(work, NULL, (xmlChar*) "Organization",     org);
      xmlNewTextChild(work, NULL, (xmlChar*) "Department",       dep);
      xmlNewTextChild(work, NULL, (xmlChar*) "SubDepartment",    subdep);
      xmlNewTextChild(work, NULL, (xmlChar*) "ManagerName",      manager);
      xmlNewTextChild(work, NULL, (xmlChar*) "ManagerPhone",     mphone);
      xmlNewTextChild(work, NULL, (xmlChar*) "CollaboratorName", collaborator);
      xmlNewTextChild(work, NULL, (xmlChar*) "CollaboratorPhone", cphone);
    }
}

  
void
rubrica_save_write_notes (RubricaPersonalCard *card, xmlNodePtr xmlnode)
{
  gpointer data;

  g_return_if_fail(IS_RUBRICA_CARD(card));

  data = rubrica_personal_card_get_notes(RUBRICA_PERSONAL_CARD(card));
  if (IS_RUBRICA_NOTES(data))
    {
      xmlNodePtr notes;
      xmlNodePtr node;

      gboolean has_partner, birth, anniversary;
      xmlChar *partner, *children, *other, *pkey;
      gchar *bday, *bmonth, *byear;
      gchar *aday, *amonth, *ayear;

      birth       = rubrica_notes_know_birthday(RUBRICA_NOTES(data));
      anniversary = rubrica_notes_know_anniversary(RUBRICA_NOTES(data));

      bday   = rubrica_notes_get_birth_day(RUBRICA_NOTES(data));
      bmonth = rubrica_notes_get_anniversary_month(RUBRICA_NOTES(data));
      byear  = rubrica_notes_get_birth_year(RUBRICA_NOTES(data));
 
      aday   = rubrica_notes_get_anniversary_day (RUBRICA_NOTES(data));
      amonth = rubrica_notes_get_anniversary_month(RUBRICA_NOTES(data));
      ayear  = rubrica_notes_get_anniversary_year(RUBRICA_NOTES(data));

      g_object_get(RUBRICA_NOTES(data), "has-partner", &has_partner, 
		   "partner-name", &partner, "children", &children, 
		   "other-notes", &other, "pubkey", &pkey, NULL);
		   
      notes = xmlNewTextChild(xmlnode, NULL, (xmlChar*) "Notes", NULL);

      rubrica_io_set_bool(notes, "partner", has_partner);
      
      xmlNewTextChild(notes, NULL, (xmlChar*) "PartnerName", partner);
      xmlNewTextChild(notes, NULL, (xmlChar*) "Child",       children);
      xmlNewTextChild(notes, NULL, (xmlChar*) "OtherNotes",  other);
      xmlNewTextChild(notes, NULL, (xmlChar*) "PublicKey",   pkey);

      node = xmlNewTextChild(notes, NULL, (xmlChar*) "PartnerBirthday", NULL);
      rubrica_io_set_bool(node, "known", birth);
      rubrica_io_set_str (node, "day",   bday);
      rubrica_io_set_str (node, "month", bmonth);
      rubrica_io_set_str (node, "year",  byear);

      node = xmlNewTextChild(notes, NULL, (xmlChar*) "Anniversary", NULL);
      rubrica_io_set_bool(node, "known", anniversary);
      rubrica_io_set_str (node, "day",   aday);
      rubrica_io_set_str (node, "month", amonth);
      rubrica_io_set_str (node, "year",  ayear);
    }
}


void
rubrica_save_write_company (RubricaCompanyCard *card, xmlNodePtr xmlnode)
{
  xmlNodePtr company;
  gpointer data;
  xmlChar *name, *pi, *notes;
  
  g_object_get(RUBRICA_COMPANY_CARD(card), "company-name", &name, "pi", &pi,
	       "notes", &notes, NULL);
  
  company = xmlNewTextChild(xmlnode, NULL, (xmlChar*) "Company", NULL);
  xmlNewTextChild(company, NULL, (xmlChar*) "CompanyName", name);
  xmlNewTextChild(company, NULL, (xmlChar*) "Pi",          pi);
  xmlNewTextChild(company, NULL, (xmlChar*) "Notes",       notes);
  
  data = rubrica_company_card_get_ref(RUBRICA_COMPANY_CARD(card));
  for (; data; data = rubrica_company_card_get_next_ref(card))
    {
      xmlNodePtr xmlref;
      xmlChar *info;
      gchar *id;

      g_object_get(RUBRICA_REF(data), "id", &id, "ref-info", &info, NULL);

      xmlref = xmlNewTextChild(company, NULL, (xmlChar*) "Refs", info);
      rubrica_io_set_str(xmlref, "refid", id);      
    }
}







