/*
*  RAL -- Rubrica Addressbook Library
*  file: telephone.h
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#include <glib.h>
#include <glib-object.h>

#include "telephone.h"
#include "utils.h"

/*   property enumetations
*/
enum {
  PHONE_NUMBER = 1,
  PHONE_TYPE
};


typedef struct _PhoneConversion {
  gint  token;
  gchar *label;
} PhoneConversion;


static PhoneConversion table[] = {
  {RUBRICA_TELEPHONE_HOME,          "home"         },
  {RUBRICA_TELEPHONE_WORK,          "work"         },
  {RUBRICA_TELEPHONE_FAX,           "fax"          },
  {RUBRICA_TELEPHONE_CELLPHONE,     "cellphone"    },
  {RUBRICA_TELEPHONE_OTHER,         "other"        },  
  {RUBRICA_TELEPHONE_OPERATOR,      "operator"     },
  {RUBRICA_TELEPHONE_GREEN,         "green"        },
  {RUBRICA_TELEPHONE_CUSTOMER_CARE, "customer care"},
  {RUBRICA_TELEPHONE_UNKNOWN,       "unknown"      }
};


struct _RubricaTelephonePrivate {
  gchar* number;
  gint   type;

  gboolean dispose_has_run;
};


static void rubrica_telephone_class_init (RubricaTelephoneClass* klass);
static void rubrica_telephone_init       (RubricaTelephone* obj);

static void rubrica_telephone_dispose    (RubricaTelephone* obj);
static void rubrica_telephone_finalize   (RubricaTelephone* obj);


static void rubrica_telephone_set_property (GObject* obj, guint property_id,
					    const GValue* value, 
					    GParamSpec* spec);
static void rubrica_telephone_get_property (GObject* obj, guint property_id,
					    GValue* value, GParamSpec* spec);

GType
rubrica_telephone_get_type()
{
  static GType tel_num_type = 0;
  
  if (!tel_num_type)
    {
      static const GTypeInfo tel_num_info =
	{
	  sizeof(RubricaTelephoneClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_telephone_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaTelephone),
	  0,
	  (GInstanceInitFunc) rubrica_telephone_init
	};

      tel_num_type = g_type_register_static (G_TYPE_OBJECT, 
					     "RubricaTelephone",
					     &tel_num_info, 0);
    }
  
  return tel_num_type;
}


static void
rubrica_telephone_class_init(RubricaTelephoneClass* klass)
{
  GObjectClass *class;
  GParamSpec* pspec;
  
  class = G_OBJECT_CLASS (klass);

  class->dispose  = (GObjectFinalizeFunc) rubrica_telephone_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_telephone_finalize;

  class->set_property = rubrica_telephone_set_property;
  class->get_property = rubrica_telephone_get_property;

  /**
   * RubricaTelephone:telephone-number:
   *
   * contact's telephone number
   */
  pspec = g_param_spec_string("telephone-number",
			      "telephone number",
			      "contact's telephone number",
			      NULL,
			      G_PARAM_READWRITE);
  g_object_class_install_property(class, PHONE_NUMBER, pspec);
  
  /**
   * RubricaTelephone:telephone-type:
   *
   * telephone number's type. See #RubricaTelephoneType for valid values
   */
  pspec = g_param_spec_int("telephone-type",
			   "telephone number's type",
			   "telephone number's type",
			   RUBRICA_TELEPHONE_HOME,        // min
			   RUBRICA_TELEPHONE_UNKNOWN,     // max
			   RUBRICA_TELEPHONE_UNKNOWN,     // default
			   G_PARAM_READWRITE);
  g_object_class_install_property(class, PHONE_TYPE, pspec);
}


static void
rubrica_telephone_init(RubricaTelephone* self)
{
  self->private = g_new(RubricaTelephonePrivate, 1);

  self->private->number = NULL;

  self->private->dispose_has_run = FALSE;
}


static void 
rubrica_telephone_dispose (RubricaTelephone* self)
{
  g_return_if_fail(IS_RUBRICA_TELEPHONE(self));
  
#ifdef DEBUG
  g_print("\n\t\trubrica_telephone_dispose");
#endif

  if (self->private->dispose_has_run)
    return;

  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_telephone_finalize (RubricaTelephone* self)
{
  g_return_if_fail(IS_RUBRICA_TELEPHONE(self));

#ifdef DEBUG
  g_print("\n\t\trubrica_telephone_finalize");
#endif

  rubrica_utils_free_string(self->private->number);

  g_free(self->private);
  self->private = NULL;
}



static void 
rubrica_telephone_set_property (GObject* obj, guint property_id,
				const GValue* value, GParamSpec* spec)
{
  RubricaTelephone* self = (RubricaTelephone*) obj;
  
  switch (property_id)
    {
    case PHONE_NUMBER:
      g_free(self->private->number);
      self->private->number = g_value_dup_string(value);
      break;

    case PHONE_TYPE:
      self->private->type = g_value_get_int(value);
      break;

    default:
      break;
    }
}


static void 
rubrica_telephone_get_property (GObject* obj, guint property_id,
				GValue* value, GParamSpec* spec)
{
  RubricaTelephone* self = (RubricaTelephone*) obj;
  
  switch (property_id)
    {
    case PHONE_NUMBER:
      g_value_set_string(value, self->private->number);
      break;

    case PHONE_TYPE:
      g_value_set_int(value, self->private->type);
      break;

    default:
      break;
    }
}


/**
 * rubrica_telephone_new
 * 
 * create a new #RubricaTelephone
 *
 * returns: a #RubricaTelephone*
 */
RubricaTelephone*
rubrica_telephone_new(void)
{
  RubricaTelephone* telephone;

  telephone = g_object_new(rubrica_telephone_get_type(), NULL);

  return telephone;
}


/**
 * rubrica_telephone_free
 * @telephone: a #RubricaTelephone
 *
 * free the #RubricaTelephone*
 */
void
rubrica_telephone_free(RubricaTelephone* telephone)
{
  g_return_if_fail(IS_RUBRICA_TELEPHONE(telephone));

#ifdef DEBUG
  g_print("\n\t\t**** Free Telephone");
#endif

  g_object_unref(telephone);   
}


/**
 * rubrica_telephone_get_number
 * @telephone: a #RubricaTelephone
 * 
 * copy the given telephone 
 *
 * returns: a new #RubricaTelephone* 
 */
RubricaTelephone* 
rubrica_telephone_copy (RubricaTelephone *telephone)
{
  RubricaTelephone* new;
  RubricaTelephoneType type;
  gchar *number;

  g_return_val_if_fail(IS_RUBRICA_TELEPHONE(telephone), NULL);

  new = rubrica_telephone_new();

  g_object_get(G_OBJECT(telephone), "telephone-number", &number, 
	       "telephone-type", &type, NULL); 
  g_object_set(G_OBJECT(new), "telephone-number", number, 
	       "telephone-type", type, NULL);

  return new;
}



/**
 * rubrica_telephone_decode_type
 * @type: a #RubricaTelephoneType
 *
 * decode the given type
 *
 * returns: a gchar*
 */
gchar* 
rubrica_telephone_decode_type (RubricaTelephoneType type)
{
  if ((type < RUBRICA_TELEPHONE_HOME) || (type > RUBRICA_TELEPHONE_UNKNOWN))
    return "unknown";  

  return table[type].label;
}

/**
 * rubrica_telephone_decode_type
 * @type: a gchar*
 *
 * encode the given string 
 *
 * returns: a #RubricaTelephoneType 
 */
RubricaTelephoneType 
rubrica_telephone_encode_type (gchar* type)
{
  g_return_val_if_fail(type != NULL, RUBRICA_TELEPHONE_UNKNOWN);

  if ((g_ascii_strcasecmp(type, "home") == 0))
    return RUBRICA_TELEPHONE_HOME;
  
  if ((g_ascii_strcasecmp(type, "work") == 0))
    return RUBRICA_TELEPHONE_WORK;

  if ((g_ascii_strcasecmp(type, "fax") == 0))
    return RUBRICA_TELEPHONE_FAX;
  
  if ((g_ascii_strcasecmp(type, "cellphone") == 0))
    return RUBRICA_TELEPHONE_CELLPHONE;

  if ((g_ascii_strcasecmp(type, "other") == 0))
    return RUBRICA_TELEPHONE_OTHER;

  if ((g_ascii_strcasecmp(type, "operator") == 0))
    return RUBRICA_TELEPHONE_OPERATOR;
  
  if ((g_ascii_strcasecmp(type, "green") == 0))
    return RUBRICA_TELEPHONE_GREEN;
  
  if ((g_ascii_strcasecmp(type, "customer care") == 0))
    return RUBRICA_TELEPHONE_CUSTOMER_CARE;
  
  return RUBRICA_TELEPHONE_UNKNOWN;
}
