/*
*  RAL -- Rubrica Addressbook Library
*  file: timeout.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <glib.h>
#include <glib-object.h>

#include "timeout.h"


struct _RubricaTimeoutPrivate {
  gint timeout;
  gboolean have_timeout;

  gboolean dispose_has_run;
};

static void rubrica_timeout_class_init (RubricaTimeoutClass* klass);
static void rubrica_timeout_init       (RubricaTimeout* obj);

static void rubrica_timeout_dispose    (RubricaTimeout* obj);
static void rubrica_timeout_finalize   (RubricaTimeout* obj);

static gboolean rubrica_timeout_emit_signal (RubricaTimeout *timeout);


/*    signals enumeration 
 */
enum {
  TIMEOUT,                 /* signal: timeout, countdown is done */ 
  LAST_SIGNAL
};


static guint rubrica_timeout_signals[LAST_SIGNAL] = {0};



GType
rubrica_timeout_get_type()
{
  static GType rubrica_timeout_type = 0;
  
  if (!rubrica_timeout_type)
    {
      static const GTypeInfo rubrica_timeout_info =
	{
	  sizeof(RubricaTimeoutClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_timeout_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaTimeout),
	  0,
	  (GInstanceInitFunc) rubrica_timeout_init
	};
      
      rubrica_timeout_type = g_type_register_static (G_TYPE_OBJECT,
						     "RubricaTimeout",
						     &rubrica_timeout_info,
						     0);	
    }
  
  return rubrica_timeout_type;
}


static void
rubrica_timeout_class_init(RubricaTimeoutClass* klass)
{
  GObjectClass *class;
  
  class = G_OBJECT_CLASS (klass);
  
  class->dispose  = (GObjectFinalizeFunc) rubrica_timeout_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_timeout_finalize;
  
  rubrica_timeout_signals[TIMEOUT] = g_signal_new("timeout", 
	       RUBRICA_TIMEOUT_TYPE,
	       G_SIGNAL_RUN_LAST,
	       G_STRUCT_OFFSET(RubricaTimeoutClass, timeout_signal),
	       NULL, 
	       NULL,
	       g_cclosure_marshal_VOID__VOID,
	       G_TYPE_NONE,
	       0,
	       G_TYPE_NONE);
}


static void
rubrica_timeout_init(RubricaTimeout* timeout)
{  
  timeout->private = g_new(RubricaTimeoutPrivate, 1);
  
  timeout->private->timeout      = 0;
  timeout->private->have_timeout = FALSE;
  
  timeout->private->dispose_has_run = FALSE;
}


static void 
rubrica_timeout_dispose (RubricaTimeout* timeout)
{
  g_return_if_fail(IS_RUBRICA_TIMEOUT(timeout));
  
  if (timeout->private->dispose_has_run)
    return;
  
  timeout->private->dispose_has_run = TRUE;
}


static void 
rubrica_timeout_finalize (RubricaTimeout* timeout)
{
  g_return_if_fail(IS_RUBRICA_TIMEOUT(timeout));
  
  g_free(timeout->private);
  timeout->private = NULL;
}


static gboolean
rubrica_timeout_emit_signal(RubricaTimeout *timeout)
{
  g_return_val_if_fail(IS_RUBRICA_TIMEOUT(timeout), FALSE);
  
  if (timeout->private->have_timeout)
    {
      g_signal_emit_by_name(timeout, "timeout", G_TYPE_NONE);
      
      return TRUE;
    }
  
  return FALSE;
}



/* Public Functions
 */


/**
 * rubrica_timeout_new
 * @time: time
 *
 * Create a new timer. 
 * Timer will wait "time" seconds, when a "timeout" signal is emitted.
 * If time is 0, then timer will wait 15 minutes, before emit signal.
 *
 * returns: a #RubricaTimeout*
 */
RubricaTimeout* 
rubrica_timeout_new(guint32 time)
{
  RubricaTimeout* timeout;  

  timeout = g_object_new(rubrica_timeout_get_type(), NULL);
  timeout->wait_time = time;

  return (RubricaTimeout*) timeout;
}

/**
 * rubrica_timeout_free
 * @timeout: a #RubricaTimeout
 *
 * free the #RubricaTimeout object
 */
void 
rubrica_timeout_free(RubricaTimeout *timeout)
{
  g_return_if_fail(IS_RUBRICA_TIMEOUT(timeout));
  
  g_object_unref(timeout);
}


/**
 * rubrica_timeout_start
 * @timeout: #RubricaTimeout
 *
 * start the timeout object
 */
void 
rubrica_timeout_start(RubricaTimeout *timeout)
{
  g_return_if_fail(IS_RUBRICA_TIMEOUT(timeout));

  /*     default time to wait is 15 minutes 
   */
  if (timeout->wait_time == 0)
    timeout->wait_time = 15;  
  
  /*     wait_time *= 60000 (1 minute = 60 seconds * 1000 microseconds)
  */

  timeout->wait_time *= 60000;
  timeout->private->timeout = 
    g_timeout_add(timeout->wait_time, 
		  (GSourceFunc) rubrica_timeout_emit_signal, timeout);
  
  timeout->private->have_timeout = TRUE;
}


/**
 * rubrica_timeout_enable
 * @timeout: a #RubricaTimeout
 *
 * enable the timeout
 */
void 
rubrica_timeout_enable(RubricaTimeout *timeout)
{
  g_return_if_fail(IS_RUBRICA_TIMEOUT(timeout));

  rubrica_timeout_start(timeout);  
}


/**
 * rubrica_timeout_disable
 * @timeout: a #RubricaTimeout
 *
 * disable the timeout
 */
void 
rubrica_timeout_disable(RubricaTimeout *timeout)
{
  g_return_if_fail(IS_RUBRICA_TIMEOUT(timeout));

  if (timeout->private->timeout != 0)
    g_source_remove(timeout->private->timeout);
  else
    return;

  timeout->private->timeout = 0;
  timeout->private->have_timeout = FALSE;
}


/**
 * rubrica_timeout_restart
 * @timeout: a #RubricaTimeout
 * @time: anew time
 *
 * restart the timeout, but waiting "time" seconds
 */
void 
rubrica_timeout_restart(RubricaTimeout *timeout, guint32 time)
{
  g_return_if_fail(IS_RUBRICA_TIMEOUT(timeout));

  rubrica_timeout_disable(timeout);

  timeout->wait_time = time;
  rubrica_timeout_enable(timeout);
}


/**
 * rubrica_timeout_is_running
 * @timeout: a #RubricaTimeout
 *
 * check if timer is running or is disabled
 *
 * returns: a gboolean, %TRUE if timeout is running, %FALSE otherwise.
 */
gboolean         
rubrica_timeout_is_running(RubricaTimeout* timeout)
{
  g_return_val_if_fail(IS_RUBRICA_TIMEOUT(timeout), FALSE);

  return timeout->private->have_timeout;
}
