/*
*  RAL -- Rubrica Addressbook Library
*  file: work.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#include <glib.h>
#include <glib-object.h>

#include "work.h"
#include "utils.h"


enum {
  WORK_ASSIGNMENT = 1,
  WORK_ORG,
  WORK_DEP,
  WORK_SUBDEP,
  WORK_MANAGER,
  WORK_MPHONE,
  WORK_COLLABORATOR,
  WORK_CPHONE
};

struct _RubricaWorkPrivate {
  gchar* assignment;            /* contact role             */
  gchar* org;                   /* organization             */
  gchar* dep;                   /* department               */
  gchar* subdep;                /* sub department           */
  gchar* manager;               /* contact's manager        */
  gchar* mphone;                /* manager telephone        */
  gchar* collaborator;          /* contact's collaborator   */
  gchar* cphone;                /* collaborator's telephone */

  gboolean dispose_has_run;
};



static void rubrica_work_class_init (RubricaWorkClass* klass);
static void rubrica_work_init       (RubricaWork* obj);

static void rubrica_work_dispose    (RubricaWork* obj);
static void rubrica_work_finalize   (RubricaWork* obj);


static void rubrica_work_set_property (GObject* obj, guint property_id,
				       const GValue* value, GParamSpec* spec);
static void rubrica_work_get_property (GObject* obj, guint property_id,
				       GValue* value, GParamSpec* spec);


GType
rubrica_work_get_type()
{
  static GType work_type = 0;
  
  if (!work_type)
    {
      static const GTypeInfo work_info =
	{
	  sizeof(RubricaWorkClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_work_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaWork),
	  0,
	  (GInstanceInitFunc) rubrica_work_init
	};

      work_type = g_type_register_static (G_TYPE_OBJECT, "RubricaWork",
					  &work_info, 0);
    }
  
  return work_type;
}


static void
rubrica_work_class_init(RubricaWorkClass* klass)
{
  GObjectClass *class;
  GParamSpec* pspec;
  
  class  = G_OBJECT_CLASS (klass);

  class->dispose  = (GObjectFinalizeFunc) rubrica_work_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_work_finalize;
  
  class->set_property = rubrica_work_set_property;
  class->get_property = rubrica_work_get_property;

  /**
   * RubricaWork:assignment
   *
   * the contact's assignment
   */
  pspec = g_param_spec_string("assignment",
			      "the contact's assignment",
			      "the contact's assignment",
			      NULL,
			      G_PARAM_READWRITE);
  g_object_class_install_property(class, WORK_ASSIGNMENT, pspec);

  /**
   * RubricaWork:organization:
   *
   * The organization the contact works for
   */
  pspec = g_param_spec_string("organization",
			      "organization",
			      "The organization the contact works for",
			      NULL,
			      G_PARAM_READWRITE);
  g_object_class_install_property(class, WORK_ORG, pspec);

  /**
   * RubricaWork:department:
   *
   * The department where contact works
   */
  pspec = g_param_spec_string("department",
			      "department",
			      "The department where contact works",
			      NULL,
			      G_PARAM_READWRITE);
  g_object_class_install_property(class, WORK_DEP, pspec);

  /**
   * RubricaWork:sub-department:
   *
   * The sub department where contact works
   */
  pspec = g_param_spec_string("sub-department",
			      "sub department",
			      "The sub department where contact works",
			      NULL,
			      G_PARAM_READWRITE);
  g_object_class_install_property(class, WORK_SUBDEP, pspec);

  /**
   * RubricaWork:manager-name:
   *
   * the contact's manager
   */
  pspec = g_param_spec_string("manager-name",
			      "contact's manager",
			      "the contact's manager ",
			      NULL,
			      G_PARAM_READWRITE);
  g_object_class_install_property(class, WORK_MANAGER, pspec);

  /**
   * RubricaWork:manager-phone:
   *
   * the manager's telephone number
   */
  pspec = g_param_spec_string("manager-phone",
			      "manager's phone",
			      "the manager's telephone number",
			      NULL,
			      G_PARAM_READWRITE);
  g_object_class_install_property(class, WORK_MPHONE, pspec);

  /**
   * RubricaWork:collaborator:
   *
   * contact's collaborator
   */
  pspec = g_param_spec_string("collaborator",
			      "contact's collaborator",
			      "contact's collaborator",
			      NULL,
			      G_PARAM_READWRITE);
  g_object_class_install_property(class, WORK_COLLABORATOR, pspec);

  /**
   * RubricaWork:collaborator-phone:
   * 
   * collaborator's phone
   */
  pspec = g_param_spec_string("collaborator-phone",
			      "collaborator's phone",
			      "collaborator's phone",
			      NULL,
			      G_PARAM_READWRITE);
  g_object_class_install_property(class, WORK_CPHONE, pspec);
}


static void
rubrica_work_init(RubricaWork* self)
{
  self->private = g_new(RubricaWorkPrivate, 1);

  self->private->assignment   = NULL;
  self->private->org          = NULL;
  self->private->dep          = NULL;
  self->private->subdep       = NULL;
  self->private->manager      = NULL;
  self->private->mphone       = NULL;
  self->private->collaborator = NULL;
  self->private->cphone       = NULL;

  self->private->dispose_has_run = FALSE;
}



static void 
rubrica_work_dispose (RubricaWork* self)
{
  g_return_if_fail(IS_RUBRICA_WORK(self));
  
#ifdef DEBUG
  g_print("\n\trubrica_work_dispose");
#endif

  if (self->private->dispose_has_run)
    return;

  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_work_finalize (RubricaWork* self)
{
  g_return_if_fail(IS_RUBRICA_WORK(self));

  rubrica_utils_free_string(self->private->assignment);
  rubrica_utils_free_string(self->private->org);
  rubrica_utils_free_string(self->private->dep);
  rubrica_utils_free_string(self->private->subdep);
  rubrica_utils_free_string(self->private->manager);
  rubrica_utils_free_string(self->private->mphone);
  rubrica_utils_free_string(self->private->collaborator);
  rubrica_utils_free_string(self->private->cphone);

  self->private->assignment = NULL;
  self->private->org = NULL;
  self->private->dep = NULL;
  self->private->subdep = NULL;
  self->private->manager = NULL;
  self->private->mphone = NULL;
  self->private->collaborator = NULL;
  self->private->cphone = NULL;  

  g_free(self->private);
  self->private = NULL;
}


static void 
rubrica_work_set_property (GObject* obj, guint property_id,
			   const GValue* value, GParamSpec* spec)
{
  RubricaWork* self = (RubricaWork*) obj;
  
  switch (property_id)
    {
    case WORK_ASSIGNMENT:
      g_free(self->private->assignment);
      self->private->assignment = g_value_dup_string(value);
      break;

    case WORK_ORG:
      g_free(self->private->org);
      self->private->org = g_value_dup_string(value);
      break;

    case WORK_DEP:
      g_free(self->private->dep);
      self->private->dep = g_value_dup_string(value);
      break;

    case WORK_SUBDEP:
      g_free(self->private->subdep);
      self->private->subdep = g_value_dup_string(value);
      break;

    case WORK_MANAGER:
      g_free(self->private->manager);
      self->private->manager = g_value_dup_string(value);
      break;

    case WORK_MPHONE:
      g_free(self->private->mphone);
      self->private->mphone = g_value_dup_string(value);
      break;

    case WORK_COLLABORATOR:
      g_free(self->private->collaborator);
      self->private->collaborator = g_value_dup_string(value);
      break;

    case WORK_CPHONE:
      g_free(self->private->cphone);
      self->private->cphone = g_value_dup_string(value);
      break;

    default:
      break;
    }
}


static void 
rubrica_work_get_property (GObject* obj, guint property_id,
			   GValue* value, GParamSpec* spec)
{
  RubricaWork* self = (RubricaWork*) obj;
  
  switch (property_id)
    {
    case WORK_ASSIGNMENT:
      g_value_set_string(value, self->private->assignment);
      break;

    case WORK_ORG:
      g_value_set_string(value, self->private->org);
      break;

    case WORK_DEP:
      g_value_set_string(value, self->private->dep);
      break;

    case WORK_SUBDEP:
      g_value_set_string(value, self->private->subdep);
      break;

    case WORK_MANAGER:
      g_value_set_string(value, self->private->manager);
      break;

    case WORK_MPHONE:
      g_value_set_string(value, self->private->mphone);
      break;

    case WORK_COLLABORATOR:
      g_value_set_string(value, self->private->collaborator);
      break;

    case WORK_CPHONE:
      g_value_set_string(value, self->private->cphone);
      break;

    default:
      break;
    }
}


/*   Public
*/


/**
 * rubrica_work_new
 *
 * create a new #RubricaWork
 *
 * returns: a new #RubricaWork*
 */
RubricaWork*
rubrica_work_new(void)
{
  RubricaWork* work;

  work = g_object_new(rubrica_work_get_type(), NULL);

  return work;
}

/**
 * rubrica_work_free
 * @work: a #RubricaWork
 *
 * free the memory owned by object
 */
void
rubrica_work_free(RubricaWork* work)
{
  g_return_if_fail(IS_RUBRICA_WORK(work));

  g_object_unref(work);   
}


/**
 * rubrica_work_copy
 * @work: a #RubricaWork
 *
 * copy the given #RubricaWork
 *
 * returns: a new allocated #RubricaWork* 
 */
RubricaWork* 
rubrica_work_copy (RubricaWork* work)
{
  RubricaWork* new;
  gchar *assignment, *org, *dep, *subdep;
  gchar *manager, *mphone, *collaborator, *cphone;

  g_return_val_if_fail(IS_RUBRICA_WORK(work), NULL);
  
  new = rubrica_work_new();

  g_object_get(G_OBJECT(work), "assignment", &assignment, 
	       "organization", &org, "department", &dep, 
	       "sub-department", &subdep, "manager-name", &manager, 
	       "manager-phone", &mphone, "collaborator", &collaborator, 
	       "collaborator-phone", &cphone, NULL);

  g_object_set(G_OBJECT(new), "assignment", assignment, 
	       "organization", org, "department", dep, 
	       "sub-department", subdep, "manager-name", manager, 
	       "manager-phone", mphone, "collaborator", collaborator, 
	       "collaborator-phone", cphone, NULL);

  return new;
}

